# distutils: libraries = flint
# distutils: depends = flint/nmod.h

################################################################################
# This file is auto-generated by the script
#   SAGE_ROOT/tools/flint-autogen/flint_autogen.py.
# From the commit 3e2c3a3e091106a25ca9c6fba28e02f2cbcd654a
# Do not modify by hand! Fix and rerun the script instead.
################################################################################

from libc.stdio cimport FILE
from sage.libs.gmp.types cimport *
from sage.libs.mpfr.types cimport *
from sage.libs.flint.types cimport *

cdef extern from "flint_wrap.h":
    void nmod_init(nmod_t * mod, mp_limb_t n) noexcept
    mp_limb_t _nmod_add(mp_limb_t a, mp_limb_t b, nmod_t mod) noexcept
    mp_limb_t nmod_add(mp_limb_t a, mp_limb_t b, nmod_t mod) noexcept
    mp_limb_t _nmod_sub(mp_limb_t a, mp_limb_t b, nmod_t mod) noexcept
    mp_limb_t nmod_sub(mp_limb_t a, mp_limb_t b, nmod_t mod) noexcept
    mp_limb_t nmod_neg(mp_limb_t a, nmod_t mod) noexcept
    mp_limb_t nmod_mul(mp_limb_t a, mp_limb_t b, nmod_t mod) noexcept
    mp_limb_t _nmod_mul_fullword(mp_limb_t a, mp_limb_t b, nmod_t mod) noexcept
    mp_limb_t nmod_inv(mp_limb_t a, nmod_t mod) noexcept
    mp_limb_t nmod_div(mp_limb_t a, mp_limb_t b, nmod_t mod) noexcept
    int nmod_divides(mp_limb_t * a, mp_limb_t b, mp_limb_t c, nmod_t mod) noexcept
    mp_limb_t nmod_pow_ui(mp_limb_t a, ulong e, nmod_t mod) noexcept
    mp_limb_t nmod_pow_fmpz(mp_limb_t a, const fmpz_t e, nmod_t mod) noexcept
    void nmod_discrete_log_pohlig_hellman_init(nmod_discrete_log_pohlig_hellman_t L) noexcept
    void nmod_discrete_log_pohlig_hellman_clear(nmod_discrete_log_pohlig_hellman_t L) noexcept
    double nmod_discrete_log_pohlig_hellman_precompute_prime(nmod_discrete_log_pohlig_hellman_t L, mp_limb_t p) noexcept
    mp_limb_t nmod_discrete_log_pohlig_hellman_primitive_root(const nmod_discrete_log_pohlig_hellman_t L) noexcept
    ulong nmod_discrete_log_pohlig_hellman_run(const nmod_discrete_log_pohlig_hellman_t L, mp_limb_t y) noexcept



r"""
Solving ordinary differential equations

This file contains functions useful for solving differential equations
which occur commonly in a 1st semester differential equations
course. For another numerical solver see the :meth:`ode_solver` function
and the optional package Octave.

Solutions from the Maxima package can contain the three constants
``_C``, ``_K1``, and ``_K2`` where the underscore is used to distinguish
them from symbolic variables that the user might have used. You can
substitute values for them, and make them into accessible usable
symbolic variables, for example with ``var("_C")``.

Commands:

- :func:`desolve` -- compute the "general solution" to a 1st or 2nd order
  ODE via Maxima

- :func:`desolve_laplace` -- solve an ODE using Laplace transforms via
  Maxima. Initial conditions are optional

- :func:`desolve_rk4` -- solve numerically an IVP for one first order
  equation, return list of points or plot

- :func:`desolve_system_rk4` -- solve numerically an IVP for a system of first
  order equations, return list of points

- :func:`desolve_odeint` -- solve numerically a system of first-order ordinary
  differential equations using :func:`~scipy:scipy.integrate.odeint` from
  the module :mod:`scipy:scipy.integrate`.

- :func:`desolve_system` -- solve a system of 1st order ODEs of any size using
  Maxima. Initial conditions are optional

- :func:`eulers_method` -- approximate solution to a 1st order DE,
  presented as a table

- :func:`eulers_method_2x2` -- approximate solution to a 1st order system
  of DEs, presented as a table

- :func:`eulers_method_2x2_plot` -- plot the sequence of points obtained
  from Euler's method

The following functions require the optional package ``tides``:

- :func:`desolve_mintides` -- numerical solution of a system of 1st order ODEs via
  the Taylor series integrator method implemented in TIDES

- :func:`desolve_tides_mpfr` -- arbitrary precision Taylor series integrator implemented in TIDES

AUTHORS:

- David Joyner (3-2006) - Initial version of functions

- Marshall Hampton (7-2007) - Creation of Python module and testing

- Robert Bradshaw (10-2008) - Some interface cleanup.

- Robert Marik (10-2009) - Some bugfixes and enhancements

- Miguel Marco (06-2014) - Tides desolvers
"""

##########################################################################
#  Copyright (C) 2006 David Joyner <wdjoyner@gmail.com>, Marshall Hampton,
#  Robert Marik <marik@mendelu.cz>
#
#  Distributed under the terms of the GNU General Public License (GPL):
#
#                  https://www.gnu.org/licenses/
##########################################################################

import os
import shutil

from sage.calculus.functional import diff
from sage.misc.lazy_import import lazy_import
lazy_import("sage.interfaces.maxima_lib","maxima")
from sage.misc.functional import N
from sage.rings.real_mpfr import RealField
from sage.structure.element import Expression


def fricas_desolve(de, dvar, ics, ivar):
    r"""
    Solve an ODE using FriCAS.

    EXAMPLES::

        sage: x = var('x')
        sage: y = function('y')(x)
        sage: desolve(diff(y,x) + y - 1, y, algorithm='fricas')             # optional - fricas
        _C0*e^(-x) + 1

        sage: desolve(diff(y, x) + y == y^3*sin(x), y, algorithm='fricas')  # optional - fricas
        -1/5*(2*cos(x)*y(x)^2 + 4*sin(x)*y(x)^2 - 5)*e^(-2*x)/y(x)^2

    TESTS::

        sage: from sage.calculus.desolvers import fricas_desolve
        sage: Y = fricas_desolve(diff(y,x) + y - 1, y, [42,1783], x)  # optional - fricas
        sage: Y.subs(x=42)                                            # optional - fricas
        1783
    """
    from sage.interfaces.fricas import fricas
    from sage.symbolic.ring import SR
    if ics is None:
        y = fricas(de).solve(dvar.operator(), ivar).sage()
    else:
        eq = fricas.equation(ivar, ics[0])
        y = fricas(de).solve(dvar.operator(), eq, ics[1:]).sage()

    if isinstance(y, dict):
        basis = y["basis"]
        particular = y["particular"]
        return particular + sum(SR.var("_C" + str(i)) * v
                                for i, v in enumerate(basis))
    return y


def fricas_desolve_system(des, dvars, ics, ivar):
    r"""
    Solve a system of first order ODEs using FriCAS.

    EXAMPLES::

        sage: t = var('t')
        sage: x = function('x')(t)
        sage: y = function('y')(t)
        sage: de1 = diff(x,t) + y - 1 == 0
        sage: de2 = diff(y,t) - x + 1 == 0
        sage: desolve_system([de1, de2], [x, y], algorithm='fricas')          # optional - fricas
        [x(t) == _C0*cos(t) + cos(t)^2 + _C1*sin(t) + sin(t)^2,
         y(t) == -_C1*cos(t) + _C0*sin(t) + 1]

        sage: desolve_system([de1, de2], [x,y], [0,1,2], algorithm='fricas')  # optional - fricas
        [x(t) == cos(t)^2 + sin(t)^2 - sin(t), y(t) == cos(t) + 1]

    TESTS::

        sage: from sage.calculus.desolvers import fricas_desolve_system
        sage: t = var('t')
        sage: x = function('x')(t)
        sage: fricas_desolve_system([diff(x,t) + 1 == 0], [x], None, t)   # optional - fricas
        [x(t) == _C0 - t]

        sage: y = function('y')(t)
        sage: de1 = diff(x,t) + y - 1 == 0
        sage: de2 = diff(y,t) - x + 1 == 0
        sage: sol = fricas_desolve_system([de1,de2], [x,y], [0,1,-1], t)  # optional - fricas
        sage: sol                                                         # optional - fricas
        [x(t) == cos(t)^2 + sin(t)^2 + 2*sin(t), y(t) == -2*cos(t) + 1]
    """
    from sage.interfaces.fricas import fricas
    from sage.symbolic.relation import solve
    from sage.symbolic.ring import SR
    ops = [dvar.operator() for dvar in dvars]
    y = fricas(des).solve(ops, ivar).sage()
    basis = y["basis"]
    particular = y["particular"]
    pars = [SR.var("_C" + str(i)) for i in range(len(basis))]
    solv = particular + sum(p * v for p, v in zip(pars, basis))

    if ics is None:
        sols = solv
    else:
        ics0 = ics[0]
        eqs = [val == sol.subs({ivar: ics0}) for val, sol in zip(ics[1:], solv)]
        pars_values = solve(eqs, pars, solution_dict=True)
        sols = [sol.subs(pars_values[0]) for sol in solv]

    return [dvar == sol for dvar, sol in zip(dvars, sols)]


def desolve(de, dvar, ics=None, ivar=None, show_method=False, contrib_ode=False,
            algorithm='maxima'):
    r"""
    Solve a 1st or 2nd order linear ODE, including IVP and BVP.

    INPUT:

    - ``de`` -- an expression or equation representing the ODE

    - ``dvar`` -- the dependent variable (hereafter called `y`)

    - ``ics`` -- (optional) the initial or boundary conditions

      - for a first-order equation, specify the initial `x` and `y`

      - for a second-order equation, specify the initial `x`, `y`,
        and `dy/dx`, i.e. write `[x_0, y(x_0), y'(x_0)]`

      - for a second-order boundary solution, specify initial and
        final `x` and `y` boundary conditions, i.e. write `[x_0, y(x_0), x_1, y(x_1)]`.

      - gives an error if the solution is not SymbolicEquation (as happens for
        example for a Clairaut equation)

    - ``ivar`` -- (optional) the independent variable (hereafter called
      `x`), which must be specified if there is more than one
      independent variable in the equation

    - ``show_method`` -- (optional) if ``True``, then Sage returns pair
      ``[solution, method]``, where method is the string describing
      the method which has been used to get a solution (Maxima uses the
      following order for first order equations: linear, separable,
      exact (including exact with integrating factor), homogeneous,
      bernoulli, generalized homogeneous) - use carefully in class,
      see below the example of an equation which is separable but
      this property is not recognized by Maxima and the equation is solved
      as exact.

    - ``contrib_ode`` -- (optional) if ``True``, ``desolve`` allows to solve
      Clairaut, Lagrange, Riccati and some other equations. This may take
      a long time and is thus turned off by default.  Initial conditions
      can be used only if the result is one SymbolicEquation (does not
      contain a singular solution, for example).

    - ``algorithm`` -- (default: ``'maxima'``) one of

      * ``'maxima'`` -- use maxima
      * ``'fricas'`` -- use FriCAS (the optional fricas spkg has to be installed)

    OUTPUT:

    In most cases return a SymbolicEquation which defines the solution
    implicitly.  If the result is in the form `y(x)=\ldots` (happens for
    linear eqs.), return the right-hand side only. The possible
    constant solutions of separable ODEs are omitted.

    .. NOTE::

        Use ``desolve? <tab>`` if the output in the Sage notebook is truncated.

    EXAMPLES::

        sage: x = var('x')
        sage: y = function('y')(x)
        sage: desolve(diff(y,x) + y - 1, y)
        (_C + e^x)*e^(-x)

    ::

        sage: f = desolve(diff(y,x) + y - 1, y, ics=[10,2]); f
        (e^10 + e^x)*e^(-x)

    ::

        sage: plot(f)
        Graphics object consisting of 1 graphics primitive

    We can also solve second-order differential equations::

        sage: x = var('x')
        sage: y = function('y')(x)
        sage: de = diff(y,x,2) - y == x
        sage: desolve(de, y)
        _K2*e^(-x) + _K1*e^x - x

    ::

        sage: f = desolve(de, y, [10,2,1]); f
        -x + 7*e^(x - 10) + 5*e^(-x + 10)

    ::

        sage: f(x=10)
        2

    ::

        sage: diff(f,x)(x=10)
        1

    ::

        sage: de = diff(y,x,2) + y == 0
        sage: desolve(de, y)
        _K2*cos(x) + _K1*sin(x)

    ::

        sage: desolve(de, y, [0,1,pi/2,4])
        cos(x) + 4*sin(x)

    ::

        sage: desolve(y*diff(y,x)+sin(x)==0,y)
        -1/2*y(x)^2 == _C - cos(x)

    Clairaut equation: general and singular solutions::

        sage: desolve(diff(y,x)^2+x*diff(y,x)-y==0,y,contrib_ode=True,show_method=True)
        [[y(x) == _C^2 + _C*x, y(x) == -1/4*x^2], 'clairau...']

    For equations involving more variables we specify an independent variable::

        sage: a,b,c,n=var('a b c n')
        sage: desolve(x^2*diff(y,x)==a+b*x^n+c*x^2*y^2,y,ivar=x,contrib_ode=True)
        [[y(x) == 0, (b*x^(n - 2) + a/x^2)*c^2*u == 0]]

    ::

        sage: desolve(x^2*diff(y,x)==a+b*x^n+c*x^2*y^2,y,ivar=x,contrib_ode=True,show_method=True)
        [[[y(x) == 0, (b*x^(n - 2) + a/x^2)*c^2*u == 0]], 'riccati']


    Higher order equations, not involving independent variable::

        sage: desolve(diff(y,x,2)+y*(diff(y,x,1))^3==0,y).expand()
        1/6*y(x)^3 + _K1*y(x) == _K2 + x

    ::

        sage: desolve(diff(y,x,2)+y*(diff(y,x,1))^3==0,y,[0,1,1,3]).expand()
        1/6*y(x)^3 - 5/3*y(x) == x - 3/2

    ::

        sage: desolve(diff(y,x,2)+y*(diff(y,x,1))^3==0,y,[0,1,1,3],show_method=True)
        [1/6*y(x)^3 - 5/3*y(x) == x - 3/2, 'freeofx']

    Separable equations - Sage returns solution in implicit form::

        sage: desolve(diff(y,x)*sin(y) == cos(x),y)
        -cos(y(x)) == _C + sin(x)

    ::

        sage: desolve(diff(y,x)*sin(y) == cos(x),y,show_method=True)
        [-cos(y(x)) == _C + sin(x), 'separable']

    ::

        sage: desolve(diff(y,x)*sin(y) == cos(x),y,[pi/2,1])
        -cos(y(x)) == -cos(1) + sin(x) - 1

    Linear equation - Sage returns the expression on the right hand side only::

        sage: desolve(diff(y,x)+(y) == cos(x),y)
        1/2*((cos(x) + sin(x))*e^x + 2*_C)*e^(-x)

    ::

        sage: desolve(diff(y,x)+(y) == cos(x),y,show_method=True)
        [1/2*((cos(x) + sin(x))*e^x + 2*_C)*e^(-x), 'linear']

    ::

        sage: desolve(diff(y,x)+(y) == cos(x),y,[0,1])
        1/2*(cos(x)*e^x + e^x*sin(x) + 1)*e^(-x)

    This ODE with separated variables is solved as
    exact. Explanation - factor does not split `e^{x-y}` in Maxima
    into `e^{x}e^{y}`::

        sage: desolve(diff(y,x)==exp(x-y),y,show_method=True)
        [-e^x + e^y(x) == _C, 'exact']

    You can solve Bessel equations, also using initial
    conditions, but you cannot put (sometimes desired) the initial
    condition at `x=0`, since this point is a singular point of the
    equation. Anyway, if the solution should be bounded at `x=0`, then
    ``_K2=0``. ::

        sage: desolve(x^2*diff(y,x,x)+x*diff(y,x)+(x^2-4)*y==0,y)
        _K1*bessel_J(2, x) + _K2*bessel_Y(2, x)

    Example of difficult ODE producing an error::

        sage: desolve(sqrt(y)*diff(y,x)+e^(y)+cos(x)-sin(x+y)==0,y) # not tested
        Traceback (click to the left for traceback)
        ...
        NotImplementedError, "Maxima was unable to solve this ODE. Consider to set option contrib_ode to True."

    Another difficult ODE with error - moreover, it takes a long time::

        sage: desolve(sqrt(y)*diff(y,x)+e^(y)+cos(x)-sin(x+y)==0,y,contrib_ode=True) # not tested

    Some more types of ODEs::

        sage: desolve(x*diff(y,x)^2-(1+x*y)*diff(y,x)+y==0,y,contrib_ode=True,show_method=True)
        [[y(x) == _C + log(x), y(x) == _C*e^x], 'factor']

    ::

        sage: desolve(diff(y,x)==(x+y)^2,y,contrib_ode=True,show_method=True)
        [[[x == _C - arctan(sqrt(t)), y(x) == -x - sqrt(t)], [x == _C + arctan(sqrt(t)), y(x) == -x + sqrt(t)]], 'lagrange']

    These two examples produce an error (as expected, Maxima 5.18 cannot
    solve equations from initial conditions). Maxima 5.18
    returns false answer in this case! ::

        sage: desolve(diff(y,x,2)+y*(diff(y,x,1))^3==0,y,[0,1,2]).expand() # not tested
        Traceback (click to the left for traceback)
        ...
        NotImplementedError, "Maxima was unable to solve this ODE. Consider to set option contrib_ode to True."

    ::

        sage: desolve(diff(y,x,2)+y*(diff(y,x,1))^3==0,y,[0,1,2],show_method=True) # not tested
        Traceback (click to the left for traceback)
        ...
        NotImplementedError, "Maxima was unable to solve this ODE. Consider to set option contrib_ode to True."

    Second order linear ODE::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == cos(x),y)
        (_K2*x + _K1)*e^(-x) + 1/2*sin(x)

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == cos(x),y,show_method=True)
        [(_K2*x + _K1)*e^(-x) + 1/2*sin(x), 'variationofparameters']

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == cos(x),y,[0,3,1])
        1/2*(7*x + 6)*e^(-x) + 1/2*sin(x)

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == cos(x),y,[0,3,1],show_method=True)
        [1/2*(7*x + 6)*e^(-x) + 1/2*sin(x), 'variationofparameters']

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == cos(x),y,[0,3,pi/2,2])
        3*(x*(e^(1/2*pi) - 2)/pi + 1)*e^(-x) + 1/2*sin(x)

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == cos(x),y,[0,3,pi/2,2],show_method=True)
        [3*(x*(e^(1/2*pi) - 2)/pi + 1)*e^(-x) + 1/2*sin(x), 'variationofparameters']

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == 0,y)
        (_K2*x + _K1)*e^(-x)

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == 0,y,show_method=True)
        [(_K2*x + _K1)*e^(-x), 'constcoeff']

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == 0,y,[0,3,1])
        (4*x + 3)*e^(-x)

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == 0,y,[0,3,1],show_method=True)
        [(4*x + 3)*e^(-x), 'constcoeff']

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == 0,y,[0,3,pi/2,2])
        (2*x*(2*e^(1/2*pi) - 3)/pi + 3)*e^(-x)

    ::

        sage: desolve(diff(y,x,2)+2*diff(y,x)+y == 0,y,[0,3,pi/2,2],show_method=True)
        [(2*x*(2*e^(1/2*pi) - 3)/pi + 3)*e^(-x), 'constcoeff']

    Using ``algorithm='fricas'`` we can invoke the differential
    equation solver from FriCAS.  For example, it can solve higher
    order linear equations::

        sage: de = x^3*diff(y, x, 3) + x^2*diff(y, x, 2) - 2*x*diff(y, x) + 2*y - 2*x^4
        sage: Y = desolve(de, y, algorithm='fricas'); Y               # optional - fricas
        (2*x^3 - 3*x^2 + 1)*_C0/x + (x^3 - 1)*_C1/x
         + (x^3 - 3*x^2 - 1)*_C2/x + 1/15*(x^5 - 10*x^3 + 20*x^2 + 4)/x

    The initial conditions are then interpreted as `[x_0, y(x_0),
    y'(x_0), \ldots, y^(n)(x_0)]`::

        sage: Y = desolve(de, y, ics=[1,3,7], algorithm='fricas'); Y  # optional - fricas
        1/15*(x^5 + 15*x^3 + 50*x^2 - 21)/x

    FriCAS can also solve some non-linear equations::

        sage: de = diff(y, x) == y / (x+y*log(y))
        sage: Y = desolve(de, y, algorithm='fricas'); Y               # optional - fricas
        1/2*(log(y(x))^2*y(x) - 2*x)/y(x)

    TESTS:

    :issue:`9961` fixed (allow assumptions on the dependent variable in desolve)::

        sage: y=function('y')(x); assume(x>0); assume(y>0)
        sage: sage.calculus.calculus.maxima('domain:real')  # needed since Maxima 5.26.0 to get the answer as below
        real
        sage: desolve(x*diff(y,x)-x*sqrt(y^2+x^2)-y == 0, y, contrib_ode=True)
        [x - arcsinh(y(x)/x) == _C]

    :issue:`10682` updated Maxima to 5.26, and it started to show a different
    solution in the complex domain for the ODE above::

        sage: forget()
        sage: sage.calculus.calculus.maxima('domain:complex')  # back to the default complex domain
        complex
        sage: assume(x>0)
        sage: assume(y>0)
        sage: desolve(x*diff(y,x)-x*sqrt(y^2+x^2)-y == 0, y, contrib_ode=True)
        [x - arcsinh(y(x)^2/(x*sqrt(y(x)^2))) - arcsinh(y(x)/x) + 1/2*log(4*(x^2 + 2*y(x)^2 + 2*sqrt(x^2*y(x)^2 + y(x)^4))/x^2) == _C]

    :issue:`6479` fixed::

        sage: x = var('x')
        sage: y = function('y')(x)
        sage: desolve( diff(y,x,x) == 0, y, [0,0,1])
        x

    ::

        sage: desolve( diff(y,x,x) == 0, y, [0,1,1])
        x + 1

    :issue:`9835` fixed::

        sage: x = var('x')
        sage: y = function('y')(x)
        sage: desolve(diff(y,x,2)+y*(1-y^2)==0,y,[0,-1,1,1])
        Traceback (most recent call last):
        ...
        NotImplementedError: Unable to use initial condition for this equation (freeofx).

    :issue:`8931` fixed::

        sage: x=var('x'); f=function('f')(x); k=var('k'); assume(k>0)
        sage: desolve(diff(f,x,2)/f==k,f,ivar=x)
        _K1*e^(sqrt(k)*x) + _K2*e^(-sqrt(k)*x)

    :issue:`15775` fixed::

        sage: forget()
        sage: y = function('y')(x)
        sage: desolve(diff(y, x) == sqrt(abs(y)), dvar=y, ivar=x)
        sqrt(-y(x))*(sgn(y(x)) - 1) + (sgn(y(x)) + 1)*sqrt(y(x)) == _C + x

    AUTHORS:

    - David Joyner (1-2006)
    - Robert Bradshaw (10-2008)
    - Robert Marik (10-2009)
    """
    if isinstance(de, Expression) and de.is_relational():
        de = de.lhs() - de.rhs()
    if isinstance(dvar, Expression) and dvar.is_symbol():
        raise ValueError("You have to declare dependent variable as a function evaluated at the independent variable, eg. y=function('y')(x)")
    # for backwards compatibility
    if isinstance(dvar, list):
        dvar, ivar = dvar
    elif ivar is None:
        ivars = de.variables()
        ivars = [t for t in ivars if t is not dvar]
        if len(ivars) != 1:
            raise ValueError("Unable to determine independent variable, please specify.")
        ivar = ivars[0]

    if algorithm == "fricas":
        return fricas_desolve(de, dvar, ics, ivar)
    if algorithm != "maxima":
        raise ValueError("unknown algorithm %s" % algorithm)

    de00 = de._maxima_()
    P = de00.parent()
    dvar_str = P(dvar.operator()).str()
    ivar_str = P(ivar).str()
    de00 = de00.str()

    def sanitize_var(exprs):
        return exprs.replace("'" + dvar_str + "(" + ivar_str + ")", dvar_str)
    de0 = sanitize_var(de00)
    ode_solver = "ode2"
    cmd = "(TEMP:%s(%s,%s,%s), if TEMP=false then TEMP else substitute(%s=%s(%s),TEMP))" % (ode_solver, de0, dvar_str, ivar_str, dvar_str, dvar_str, ivar_str)
    # we produce string like this
    # ode2('diff(y,x,2)+2*'diff(y,x,1)+y-cos(x),y(x),x)
    soln = P(cmd)

    if str(soln).strip() == 'false':
        if contrib_ode:
            ode_solver = "contrib_ode"
            P("load('contrib_ode)")
            cmd = "(TEMP:%s(%s,%s,%s), if TEMP=false then TEMP else substitute(%s=%s(%s),TEMP))" % (ode_solver, de0, dvar_str, ivar_str, dvar_str, dvar_str, ivar_str)
            # we produce string like this
            # (TEMP:contrib_ode(x*('diff(y,x,1))^2-(x*y+1)*'diff(y,x,1)+y,y,x), if TEMP=false then TEMP else substitute(y=y(x),TEMP))
            soln = P(cmd)
            if str(soln).strip() == 'false':
                raise NotImplementedError("Maxima was unable to solve this ODE.")
        else:
            raise NotImplementedError("Maxima was unable to solve this ODE. Consider to set option contrib_ode to True.")

    if show_method:
        maxima_method = P("method")

    if ics is not None:
        if not (isinstance(soln.sage(), Expression) and soln.sage().is_relational()):
            if not show_method:
                maxima_method = P("method")
            raise NotImplementedError("Unable to use initial condition for this equation (%s)." % (str(maxima_method).strip()))
        if len(ics) == 2:
            tempic = (ivar == ics[0])._maxima_().str()
            tempic = tempic + "," + (dvar == ics[1])._maxima_().str()
            cmd = "(TEMP:ic1(%s(%s,%s,%s),%s),substitute(%s=%s(%s),TEMP))" % (ode_solver, de00, dvar_str, ivar_str, tempic, dvar_str, dvar_str, ivar_str)
            cmd = sanitize_var(cmd)
            # we produce string like this
            # (TEMP:ic2(ode2('diff(y,x,2)+2*'diff(y,x,1)+y-cos(x),y,x),x=0,y=3,'diff(y,x)=1),substitute(y=y(x),TEMP))
            soln = P(cmd)
        if len(ics) == 3:
            # fixed ic2 command from Maxima - we have to ensure that %k1, %k2 do not depend on variables, should be removed when fixed in Maxima
            P("ic2_sage(soln,xa,ya,dya):=block([programmode:true,backsubst:true,singsolve:true,temp,%k2,%k1,TEMP_k], \
                noteqn(xa), noteqn(ya), noteqn(dya), boundtest('%k1,%k1), boundtest('%k2,%k2), \
                temp: lhs(soln) - rhs(soln), \
                TEMP_k:solve([subst([xa,ya],soln), subst([dya,xa], lhs(dya)=-subst(0,lhs(dya),diff(temp,lhs(xa)))/diff(temp,lhs(ya)))],[%k1,%k2]), \
                if not freeof(lhs(ya),TEMP_k) or not freeof(lhs(xa),TEMP_k) then return (false), \
                temp: maplist(lambda([zz], subst(zz,soln)), TEMP_k), \
                if length(temp)=1 then return(first(temp)) else return(temp))")
            tempic = P(ivar == ics[0]).str()
            tempic += "," + P(dvar == ics[1]).str()
            tempic += ",'diff(" + dvar_str + "," + ivar_str + ")=" + P(ics[2]).str()
            cmd = "(TEMP:ic2_sage(%s(%s,%s,%s),%s),substitute(%s=%s(%s),TEMP))" % (ode_solver, de00, dvar_str, ivar_str, tempic, dvar_str, dvar_str, ivar_str)
            cmd = sanitize_var(cmd)
            # we produce string like this
            # (TEMP:ic2(ode2('diff(y,x,2)+2*'diff(y,x,1)+y-cos(x),y,x),x=0,y=3,'diff(y,x)=1),substitute(y=y(x),TEMP))
            soln = P(cmd)
            if str(soln).strip() == 'false':
                raise NotImplementedError("Maxima was unable to solve this IVP. Remove the initial condition to get the general solution.")
        if len(ics) == 4:
            # fixed bc2 command from Maxima - we have to ensure that %k1, %k2 do not depend on variables, should be removed when fixed in Maxima
            P("bc2_sage(soln,xa,ya,xb,yb):=block([programmode:true,backsubst:true,singsolve:true,temp,%k1,%k2,TEMP_k], \
                noteqn(xa), noteqn(ya), noteqn(xb), noteqn(yb), boundtest('%k1,%k1), boundtest('%k2,%k2), \
                TEMP_k:solve([subst([xa,ya],soln), subst([xb,yb],soln)], [%k1,%k2]), \
                if not freeof(lhs(ya),TEMP_k) or not freeof(lhs(xa),TEMP_k) then return (false), \
                temp: maplist(lambda([zz], subst(zz,soln)),TEMP_k), \
                if length(temp)=1 then return(first(temp)) else return(temp))")
            cmd = "bc2_sage(%s(%s,%s,%s),%s,%s=%s,%s,%s=%s)" % (ode_solver, de00, dvar_str, ivar_str, P(ivar == ics[0]).str(), dvar_str, P(ics[1]).str(), P(ivar == ics[2]).str(), dvar_str, P(ics[3]).str())
            cmd = "(TEMP:%s,substitute(%s=%s(%s),TEMP))" % (cmd, dvar_str, dvar_str, ivar_str)
            cmd = sanitize_var(cmd)
            # we produce string like this
            # (TEMP:bc2(ode2('diff(y,x,2)+2*'diff(y,x,1)+y-cos(x),y,x),x=0,y=3,x=%pi/2,y=2),substitute(y=y(x),TEMP))
            soln = P(cmd)
            if str(soln).strip() == 'false':
                raise NotImplementedError("Maxima was unable to solve this BVP. Remove the initial condition to get the general solution.")

    soln = soln.sage()
    if isinstance(soln, Expression) and soln.is_relational() and soln.lhs() == dvar:
        # Remark: Here we do not check that the right hand side does not depend on dvar.
        # This probably will not happen for solutions obtained via ode2, anyway.
        soln = soln.rhs()
    if show_method:
        return [soln, maxima_method.str()]
    return soln


def desolve_laplace(de, dvar, ics=None, ivar=None):
    """
    Solve an ODE using Laplace transforms. Initial conditions are optional.

    INPUT:

    - ``de`` -- a lambda expression representing the ODE (e.g. ``de =
      diff(y,x,2) == diff(y,x)+sin(x)``)

    - ``dvar`` -- the dependent variable (e.g. ``y``)

    - ``ivar`` -- (optional) the independent variable (hereafter called
      `x`), which must be specified if there is more than one
      independent variable in the equation.

    - ``ics`` -- list of numbers representing initial conditions, (e.g.
      ``f(0)=1``, ``f'(0)=2`` corresponds to ``ics = [0,1,2]``)

    OUTPUT: solution of the ODE as symbolic expression

    EXAMPLES::

        sage: u=function('u')(x)
        sage: eq = diff(u,x) - exp(-x) - u == 0
        sage: desolve_laplace(eq,u)
        1/2*(2*u(0) + 1)*e^x - 1/2*e^(-x)

    We can use initial conditions::

        sage: desolve_laplace(eq,u,ics=[0,3])
        -1/2*e^(-x) + 7/2*e^x

    The initial conditions do not persist in the system (as they persisted
    in previous versions)::

        sage: desolve_laplace(eq,u)
        1/2*(2*u(0) + 1)*e^x - 1/2*e^(-x)

    ::

        sage: f=function('f')(x)
        sage: eq = diff(f,x) + f == 0
        sage: desolve_laplace(eq,f,[0,1])
        e^(-x)

    ::

        sage: x = var('x')
        sage: f = function('f')(x)
        sage: de = diff(f,x,x) - 2*diff(f,x) + f
        sage: desolve_laplace(de,f)
        -x*e^x*f(0) + x*e^x*D[0](f)(0) + e^x*f(0)

    ::

        sage: desolve_laplace(de,f,ics=[0,1,2])
        x*e^x + e^x

    TESTS:

    Check that :issue:`4839` is fixed::

        sage: t = var('t')
        sage: x = function('x')(t)
        sage: soln = desolve_laplace(diff(x,t)+x==1, x, ics=[0,2])
        sage: soln
        e^(-t) + 1

    ::

        sage: soln(t=3)
        e^(-3) + 1

    AUTHORS:

    - David Joyner (1-2006,8-2007)

    - Robert Marik (10-2009)
    """
    # This is the original code from David Joyner (inputs and outputs strings)
    # maxima("de:"+de._repr_()+"=0;")
    # if ics is not None:
    #     d = len(ics)
    #     for i in range(d-1):
    #         ic = "atvalue(diff("+vars[1]+"("+vars[0]+"),"+str(vars[0])+","+str(i)+"),"+str(vars[0])+"="+str(ics[0])+","+str(ics[1+i])+")"
    #         maxima(ic)
    #
    # cmd = "desolve("+de._repr_()+","+vars[1]+"("+vars[0]+"));"
    # return maxima(cmd).rhs()._maxima_init_()

    # verbatim copy from desolve - begin
    if isinstance(de, Expression) and de.is_relational():
        de = de.lhs() - de.rhs()
    if isinstance(dvar, Expression) and dvar.is_symbol():
        raise ValueError("You have to declare dependent variable as a function evaluated at the independent variable, eg. y=function('y')(x)")
    # for backwards compatibility
    if isinstance(dvar, list):
        dvar, ivar = dvar
    elif ivar is None:
        ivars = de.variables()
        ivars = [t for t in ivars if t is not dvar]
        if len(ivars) != 1:
            raise ValueError("Unable to determine independent variable, please specify.")
        ivar = ivars[0]
    # verbatim copy from desolve - end

    dvar_str = str(dvar)

    def sanitize_var(exprs):
        # 'y(x) -> y(x)
        return exprs.replace("'" + dvar_str, dvar_str)

    de0 = de._maxima_()
    P = de0.parent()
    i = dvar_str.find('(')
    dvar_str = dvar_str[:i + 1] + '_SAGE_VAR_' + dvar_str[i + 1:]
    cmd = sanitize_var("desolve(" + de0.str() + "," + dvar_str + ")")
    soln = P(cmd).rhs()
    if str(soln).strip() == 'false':
        raise NotImplementedError("Maxima was unable to solve this ODE.")
    soln = soln.sage()
    if ics is not None:
        d = len(ics)
        for i in range(d - 1):
            soln = eval('soln.substitute(diff(dvar,ivar,i)(' + str(ivar) + '=ics[0])==ics[i+1])')
    return soln


def desolve_system(des, vars, ics=None, ivar=None, algorithm='maxima'):
    r"""
    Solve a system of any size of 1st order ODEs. Initial conditions
    are optional.

    One dimensional systems are passed to :meth:`desolve_laplace`.

    INPUT:

    - ``des`` -- list of ODEs

    - ``vars`` -- list of dependent variables

    - ``ics`` -- (optional) list of initial values for ``ivar`` and ``vars``;
      if ``ics`` is defined, it should provide initial conditions for each
      variable, otherwise an exception would be raised

    - ``ivar`` -- (optional) the independent variable, which must be
      specified if there is more than one independent variable in the
      equation

    - ``algorithm`` -- (default: ``'maxima'``) one of

      * ``'maxima'`` -- use maxima
      * ``'fricas'`` -- use FriCAS (the optional fricas spkg has to be installed)

    EXAMPLES::

        sage: t = var('t')
        sage: x = function('x')(t)
        sage: y = function('y')(t)
        sage: de1 = diff(x,t) + y - 1 == 0
        sage: de2 = diff(y,t) - x + 1 == 0
        sage: desolve_system([de1, de2], [x,y])
        [x(t) == (x(0) - 1)*cos(t) - (y(0) - 1)*sin(t) + 1,
         y(t) == (y(0) - 1)*cos(t) + (x(0) - 1)*sin(t) + 1]

    The same system solved using FriCAS::

        sage: desolve_system([de1, de2], [x,y], algorithm='fricas')  # optional - fricas
        [x(t) == _C0*cos(t) + cos(t)^2 + _C1*sin(t) + sin(t)^2,
         y(t) == -_C1*cos(t) + _C0*sin(t) + 1]

    Now we give some initial conditions::

        sage: sol = desolve_system([de1, de2], [x,y], ics=[0,1,2]); sol
        [x(t) == -sin(t) + 1, y(t) == cos(t) + 1]

    ::

        sage: solnx, solny = sol[0].rhs(), sol[1].rhs()
        sage: plot([solnx,solny],(0,1))  # not tested
        sage: parametric_plot((solnx,solny),(0,1))  # not tested

    TESTS:

    Check that :issue:`9823` is fixed::

        sage: t = var('t')
        sage: x = function('x')(t)
        sage: de1 = diff(x,t) + 1 == 0
        sage: desolve_system([de1], [x])
        -t + x(0)

    Check that :issue:`16568` is fixed::

        sage: t = var('t')
        sage: x = function('x')(t)
        sage: y = function('y')(t)
        sage: de1 = diff(x,t) + y - 1 == 0
        sage: de2 = diff(y,t) - x + 1 == 0
        sage: des = [de1,de2]
        sage: ics = [0,1,-1]
        sage: vars = [x,y]
        sage: sol = desolve_system(des, vars, ics); sol
        [x(t) == 2*sin(t) + 1, y(t) == -2*cos(t) + 1]

    ::

        sage: solx, soly = sol[0].rhs(), sol[1].rhs()
        sage: RR(solx(t=3))
        1.28224001611973

    ::

        sage: P1 = plot([solx,soly], (0,1))                                             # needs sage.plot
        sage: P2 = parametric_plot((solx,soly), (0,1))                                  # needs sage.plot

    Now type ``show(P1)``, ``show(P2)`` to view these plots.

    Check that :issue:`9824` is fixed::

        sage: t = var('t')
        sage: epsilon = var('epsilon')
        sage: x1 = function('x1')(t)
        sage: x2 = function('x2')(t)
        sage: de1 = diff(x1,t) == epsilon
        sage: de2 = diff(x2,t) == -2
        sage: desolve_system([de1, de2], [x1, x2], ivar=t)
        [x1(t) == epsilon*t + x1(0), x2(t) == -2*t + x2(0)]
        sage: desolve_system([de1, de2], [x1, x2], ics=[1,1], ivar=t)
        Traceback (most recent call last):
        ...
        ValueError: Initial conditions aren't complete: number of vars is different
        from number of dependent variables. Got ics = [1, 1], vars = [x1(t), x2(t)]

    Check that :issue:`9825` is fixed::

        sage: t = var('t')
        sage: x1, x2=function("x1, x2")
        sage: de1=x1(t).diff(t)==-3*(x2(t)-1)
        sage: de2=x2(t).diff(t)==1
        sage: Sol=desolve_system([de1, de2],[x1(t),x2(t)],ivar=t) ; Sol
        [x1(t) == -3/2*t^2 - 3*t*x2(0) + 3*t + x1(0), x2(t) == t + x2(0)]


    AUTHORS:

    - Robert Bradshaw (10-2008)
    - Sergey Bykov (10-2014)
    """
    if ics is not None:
        if len(ics) != (len(vars) + 1):
            raise ValueError("Initial conditions aren't complete: number of vars is different from number of dependent variables. Got ics = {0}, vars = {1}".format(ics, vars))

    if len(des) == 1 and algorithm == "maxima":
        return desolve_laplace(des[0], vars[0], ics=ics, ivar=ivar)
    ivars = set()
    for i, de in enumerate(des):
        if not (isinstance(de, Expression) and de.is_relational()):
            des[i] = de == 0
        ivars = ivars.union(set(de.variables()))
    if ivar is None:
        ivars = ivars - set(vars)
        if len(ivars) != 1:
            raise ValueError("Unable to determine independent variable, please specify.")
        ivar = list(ivars)[0]

    if algorithm == "fricas":
        return fricas_desolve_system(des, vars, ics, ivar)
    if algorithm != "maxima":
        raise ValueError("unknown algorithm %s" % algorithm)

    dvars = [v._maxima_() for v in vars]
    if ics is not None:
        ivar_ic = ics[0]
        for dvar, ic in zip(dvars, ics[1:]):
            dvar.atvalue(ivar == ivar_ic, ic)
    soln = dvars[0].parent().desolve(des, dvars)
    if str(soln).strip() == 'false':
        raise NotImplementedError("Maxima was unable to solve this system.")
    soln = list(soln)
    for i, sol in enumerate(soln):
        soln[i] = sol.sage()
    if ics is not None:
        ivar_ic = ics[0]
        for dvar, ic in zip(dvars, ics[:1]):
            dvar.atvalue(ivar == ivar_ic, dvar)
    return soln


def eulers_method(f, x0, y0, h, x1, algorithm='table'):
    r"""
    This implements Euler's method for finding numerically the
    solution of the 1st order ODE `y' = f(x,y)`, `y(a)=c`. The ``x``
    column of the table increments from `x_0` to `x_1` by `h` (so
    `(x_1-x_0)/h` must be an integer). In the ``y`` column, the new
    `y`-value equals the old `y`-value plus the corresponding entry in the
    last column.

    .. NOTE::

        This function is for pedagogical purposes only.

    EXAMPLES::

        sage: from sage.calculus.desolvers import eulers_method
        sage: x,y = PolynomialRing(QQ,2,"xy").gens()
        sage: eulers_method(5*x+y-5,0,1,1/2,1)
             x                    y                  h*f(x,y)
             0                    1                   -2
           1/2                   -1                 -7/4
             1                -11/4                -11/8

    ::

        sage: x,y = PolynomialRing(QQ,2,"xy").gens()
        sage: eulers_method(5*x+y-5,0,1,1/2,1,algorithm='none')
        [[0, 1], [1/2, -1], [1, -11/4], [3/2, -33/8]]

    ::

        sage: RR = RealField(sci_not=0, prec=4, rnd='RNDU')
        sage: x,y = PolynomialRing(RR,2,"xy").gens()
        sage: eulers_method(5*x+y-5,0,1,1/2,1,algorithm="None")
        [[0, 1], [1/2, -1.0], [1, -2.7], [3/2, -4.0]]

    ::

        sage: RR = RealField(sci_not=0, prec=4, rnd='RNDU')
        sage: x,y=PolynomialRing(RR,2,"xy").gens()
        sage: eulers_method(5*x+y-5,0,1,1/2,1)
             x                    y                  h*f(x,y)
             0                    1                 -2.0
           1/2                 -1.0                 -1.7
             1                 -2.7                 -1.3

    ::

        sage: x,y=PolynomialRing(QQ,2,"xy").gens()
        sage: eulers_method(5*x+y-5,1,1,1/3,2)
                 x                    y                  h*f(x,y)
                 1                    1                  1/3
               4/3                  4/3                    1
               5/3                  7/3                 17/9
                 2                 38/9                83/27

    ::

        sage: eulers_method(5*x+y-5,0,1,1/2,1,algorithm='none')
        [[0, 1], [1/2, -1], [1, -11/4], [3/2, -33/8]]

    ::

        sage: pts = eulers_method(5*x+y-5,0,1,1/2,1,algorithm='none')
        sage: P1 = list_plot(pts)                                                       # needs sage.plot
        sage: P2 = line(pts)                                                            # needs sage.plot
        sage: (P1 + P2).show()                                                          # needs sage.plot

    AUTHORS:

    - David Joyner
    """
    if algorithm == "table":
        print("%10s %20s %25s" % ("x", "y", "h*f(x,y)"))
    n = int((1.0) * (x1 - x0) / h)
    x00 = x0
    y00 = y0
    soln = [[x00, y00]]
    for i in range(n + 1):
        if algorithm == "table":
            print("%10r %20r %20r" % (x00, y00, h * f(x00, y00)))
        y00 = y00 + h * f(x00, y00)
        x00 = x00 + h
        soln.append([x00, y00])
    if algorithm != "table":
        return soln


def eulers_method_2x2(f, g, t0, x0, y0, h, t1, algorithm='table'):
    r"""
    This implements Euler's method for finding numerically the
    solution of the 1st order system of two ODEs

    .. MATH::

        \begin{aligned}
        x' &= f(t, x, y), x(t_0)=x_0 \\
        y' &= g(t, x, y), y(t_0)=y_0.
        \end{aligned}

    The ``t`` column of the table increments from `t_0` to `t_1` by `h`
    (so `\frac{t_1-t_0}{h}` must be an integer). In the ``x`` column,
    the new `x`-value equals the old `x`-value plus the corresponding
    entry in the next (third) column.  In the ``y`` column, the new
    `y`-value equals the old `y`-value plus the corresponding entry in the
    next (last) column.

    .. NOTE::

        This function is for pedagogical purposes only.

    EXAMPLES::

        sage: from sage.calculus.desolvers import eulers_method_2x2
        sage: t, x, y = PolynomialRing(QQ,3,"txy").gens()
        sage: f = x+y+t; g = x-y
        sage: eulers_method_2x2(f,g, 0, 0, 0, 1/3, 1,algorithm='none')
        [[0, 0, 0], [1/3, 0, 0], [2/3, 1/9, 0], [1, 10/27, 1/27], [4/3, 68/81, 4/27]]

    ::

        sage: eulers_method_2x2(f,g, 0, 0, 0, 1/3, 1)
             t                    x                h*f(t,x,y)                    y           h*g(t,x,y)
             0                    0                         0                    0                    0
           1/3                    0                       1/9                    0                    0
           2/3                  1/9                      7/27                    0                 1/27
             1                10/27                     38/81                 1/27                  1/9

    ::

        sage: RR = RealField(sci_not=0, prec=4, rnd='RNDU')
        sage: t,x,y=PolynomialRing(RR,3,"txy").gens()
        sage: f = x+y+t; g = x-y
        sage: eulers_method_2x2(f,g, 0, 0, 0, 1/3, 1)
             t                    x                h*f(t,x,y)                    y           h*g(t,x,y)
             0                    0                      0.00                    0                 0.00
           1/3                 0.00                      0.13                 0.00                 0.00
           2/3                 0.13                      0.29                 0.00                0.043
             1                 0.41                      0.57                0.043                 0.15

    To numerically approximate `y(1)`, where `(1+t^2)y''+y'-y=0`,
    `y(0)=1`, `y'(0)=-1`, using 4 steps of Euler's method, first
    convert to a system: `y_1' = y_2`, `y_1(0)=1`; `y_2' =
    \frac{y_1-y_2}{1+t^2}`, `y_2(0)=-1`.::

         sage: RR = RealField(sci_not=0, prec=4, rnd='RNDU')
         sage: t, x, y=PolynomialRing(RR,3,"txy").gens()
         sage: f = y; g = (x-y)/(1+t^2)
         sage: eulers_method_2x2(f,g, 0, 1, -1, 1/4, 1)
             t                    x                h*f(t,x,y)                    y           h*g(t,x,y)
             0                    1                     -0.25                   -1                 0.50
           1/4                 0.75                     -0.12                -0.50                 0.29
           1/2                 0.63                    -0.054                -0.21                 0.19
           3/4                 0.63                   -0.0078               -0.031                 0.11
             1                 0.63                     0.020                0.079                0.071

    To numerically approximate `y(1)`, where `y''+ty'+y=0`, `y(0)=1`, `y'(0)=0`::

        sage: t,x,y=PolynomialRing(RR,3,"txy").gens()
        sage: f = y; g = -x-y*t
        sage: eulers_method_2x2(f,g, 0, 1, 0, 1/4, 1)
             t                    x                h*f(t,x,y)                    y           h*g(t,x,y)
             0                    1                      0.00                    0                -0.25
           1/4                  1.0                    -0.062                -0.25                -0.23
           1/2                 0.94                     -0.11                -0.46                -0.17
           3/4                 0.88                     -0.15                -0.62                -0.10
             1                 0.75                     -0.17                -0.68               -0.015

    AUTHORS:

    - David Joyner
    """
    if algorithm == "table":
        print("%10s %20s %25s %20s %20s" % ("t", "x", "h*f(t,x,y)", "y", "h*g(t,x,y)"))
    n = int((1.0) * (t1 - t0) / h)
    t00 = t0
    x00 = x0
    y00 = y0
    soln = [[t00, x00, y00]]
    for i in range(n + 1):
        if algorithm == "table":
            print("%10r %20r %25r %20r %20r" % (t00, x00, h * f(t00, x00, y00), y00, h * g(t00, x00, y00)))
        x01 = x00 + h * f(t00, x00, y00)
        y00 = y00 + h * g(t00, x00, y00)
        x00 = x01
        t00 = t00 + h
        soln.append([t00, x00, y00])
    if algorithm != "table":
        return soln


def eulers_method_2x2_plot(f, g, t0, x0, y0, h, t1):
    r"""
    Plot solution of ODE.

    This plots the solution in the rectangle with sides ``(xrange[0],xrange[1])`` and
    ``(yrange[0],yrange[1])``, and plots using Euler's method the
    numerical solution of the 1st order ODEs `x' = f(t,x,y)`,
    `x(a)=x_0`, `y' = g(t,x,y)`, `y(a) = y_0`.

    .. NOTE::

        This function is for pedagogical purposes only.

    EXAMPLES:

    The following example plots the solution to
    `\theta''+\sin(\theta)=0`, `\theta(0)=\frac 34`, `\theta'(0) =
    0`.  Type ``P[0].show()`` to plot the solution,
    ``(P[0]+P[1]).show()`` to plot `(t,\theta(t))` and
    `(t,\theta'(t))`::

        sage: from sage.calculus.desolvers import eulers_method_2x2_plot
        sage: f = lambda z : z[2]; g = lambda z : -sin(z[1])
        sage: P = eulers_method_2x2_plot(f,g, 0.0, 0.75, 0.0, 0.1, 1.0)                 # needs sage.plot
    """
    from sage.plot.line import line

    n = int((1.0) * (t1 - t0) / h)
    t00 = t0
    x00 = x0
    y00 = y0
    soln = [[t00, x00, y00]]
    for i in range(n + 1):
        x01 = x00 + h * f([t00, x00, y00])
        y00 = y00 + h * g([t00, x00, y00])
        x00 = x01
        t00 = t00 + h
        soln.append([t00, x00, y00])
    Q1 = line([[x[0], x[1]] for x in soln], rgbcolor=(.25, .125, .75))
    Q2 = line([[x[0], x[2]] for x in soln], rgbcolor=(.5, .125, .25))
    return [Q1, Q2]


def desolve_rk4_determine_bounds(ics, end_points=None):
    """
    Used to determine bounds for numerical integration.

    - If ``end_points`` is None, the interval for integration is from ``ics[0]``
      to ``ics[0]+10``

    - If ``end_points`` is ``a`` or ``[a]``, the interval for integration is from ``min(ics[0],a)``
      to ``max(ics[0],a)``

    - If ``end_points`` is ``[a,b]``, the interval for integration is from ``min(ics[0],a)``
      to ``max(ics[0],b)``

    EXAMPLES::

        sage: from sage.calculus.desolvers import desolve_rk4_determine_bounds
        sage: desolve_rk4_determine_bounds([0,2],1)
        (0, 1)

    ::

        sage: desolve_rk4_determine_bounds([0,2])
        (0, 10)

    ::

        sage: desolve_rk4_determine_bounds([0,2],[-2])
        (-2, 0)

    ::

        sage: desolve_rk4_determine_bounds([0,2],[-2,4])
        (-2, 4)
    """
    if end_points is None:
        return ics[0], ics[0] + 10
    if not isinstance(end_points, list):
        end_points = [end_points]
    if len(end_points) == 1:
        return min(ics[0], end_points[0]), max(ics[0], end_points[0])
    return min(ics[0], end_points[0]), max(ics[0], end_points[1])


def desolve_rk4(de, dvar, ics=None, ivar=None, end_points=None, step=0.1, output='list', **kwds):
    """
    Solve numerically one first-order ordinary differential
    equation.

    INPUT:

    Input is similar to ``desolve`` command. The differential equation can be
    written in a form close to the ``plot_slope_field`` or ``desolve`` command.

    - Variant 1 (function in two variables)

      - ``de`` -- right hand side, i.e. the function `f(x,y)` from ODE `y'=f(x,y)`

      - ``dvar`` -- dependent variable (symbolic variable declared by var)

    - Variant 2 (symbolic equation)

      - ``de`` -- equation, including term with ``diff(y,x)``

      - ``dvar`` -- dependent variable (declared as function of independent variable)

    - Other parameters

      - ``ivar`` -- should be specified, if there are more variables or if the equation is autonomous

      - ``ics`` -- initial conditions in the form ``[x0,y0]``

      - ``end_points`` -- the end points of the interval

        - if ``end_points`` is a or [a], we integrate between ``min(ics[0],a)`` and ``max(ics[0],a)``
        - if ``end_points`` is None, we use ``end_points=ics[0]+10``

        - if end_points is [a,b] we integrate between ``min(ics[0], a)`` and ``max(ics[0], b)``

      - ``step`` -- (default: 0.1) the length of the step (positive number)

      - ``output`` -- (default: ``'list'``) one of ``'list'``,
        ``'plot'``, ``'slope_field'`` (graph of the solution with slope field)

    OUTPUT:

    Return a list of points, or plot produced by ``list_plot``,
    optionally with slope field.

    .. SEEALSO::

        :func:`ode_solver`.

    EXAMPLES::

        sage: from sage.calculus.desolvers import desolve_rk4

    Variant 2 for input - more common in numerics::

        sage: x,y = var('x,y')
        sage: desolve_rk4(x*y*(2-y),y,ics=[0,1],end_points=1,step=0.5)
        [[0, 1], [0.5, 1.12419127424558], [1.0, 1.46159016228882...]]

    Variant 1 for input - we can pass ODE in the form used by
    desolve function In this example we integrate backwards, since
    ``end_points < ics[0]``::

        sage: y = function('y')(x)
        sage: desolve_rk4(diff(y,x)+y*(y-1) == x-2,y,ics=[1,1],step=0.5, end_points=0)
        [[0.0, 8.904257108962112], [0.5, 1.90932794536153...], [1, 1]]

    Here we show how to plot simple pictures. For more advanced
    applications use list_plot instead. To see the resulting picture
    use ``show(P)`` in Sage notebook. ::

        sage: x,y = var('x,y')
        sage: P=desolve_rk4(y*(2-y),y,ics=[0,.1],ivar=x,output='slope_field',end_points=[-4,6],thickness=3)

    ALGORITHM:

    `4`-th order Runge-Kutta method. Wrapper for command ``rk`` in
    Maxima's dynamics package.  Perhaps could be faster by using
    fast_float instead.

    AUTHORS:

    - Robert Marik (10-2009)
    """
    if ics is None:
        raise ValueError("No initial conditions, specify with ics=[x0,y0].")

    if output not in ['list', 'plot', 'slope_field']:
        raise ValueError("Option output should be 'list', 'plot' or 'slope_field'.")

    if ivar is None:
        ivars = de.variables()
        ivars = [t for t in ivars if t != dvar]
        if len(ivars) != 1:
            raise ValueError("Unable to determine independent variable, please specify.")
        ivar = ivars[0]

    step = abs(step)

    def desolve_rk4_inner(de, dvar):
        de0 = de._maxima_()
        maxima("load('dynamics)")
        lower_bound, upper_bound = desolve_rk4_determine_bounds(ics, end_points)
        sol_1, sol_2 = [], []
        if lower_bound < ics[0]:
            cmd = "rk(%s,%s,%s,[%s,%s,%s,%s])\
            " % (de0.str(), '_SAGE_VAR_' + str(dvar), str(ics[1]), '_SAGE_VAR_' + str(ivar), str(ics[0]), lower_bound, -step)
            sol_1 = maxima(cmd).sage()
            sol_1.pop(0)
            sol_1.reverse()
        if upper_bound > ics[0]:
            cmd = "rk(%s,%s,%s,[%s,%s,%s,%s])\
            " % (de0.str(), '_SAGE_VAR_' + str(dvar), str(ics[1]), '_SAGE_VAR_' + str(ivar), str(ics[0]), upper_bound, step)
            sol_2 = maxima(cmd).sage()
            sol_2.pop(0)
        sol = sol_1
        sol.extend([[ics[0], ics[1]]])
        sol.extend(sol_2)

        if output == 'list':
            return sol
        from sage.plot.plot import list_plot
        from sage.plot.plot_field import plot_slope_field
        R = list_plot(sol, plotjoined=True, **kwds)
        if output == 'plot':
            return R
        if output == 'slope_field':
            XMIN = sol[0][0]
            YMIN = sol[0][1]
            XMAX = XMIN
            YMAX = YMIN
            for s, t in sol:
                XMAX = max(s, XMAX)
                XMIN = min(s, XMIN)
                YMAX = max(t, YMAX)
                YMIN = min(t, YMIN)
            return plot_slope_field(de, (ivar, XMIN, XMAX), (dvar, YMIN, YMAX)) + R

    if not (isinstance(dvar, Expression) and dvar.is_symbol()):
        from sage.calculus.functional import diff
        from sage.symbolic.relation import solve
        from sage.symbolic.ring import SR
        if isinstance(de, Expression) and de.is_relational():
            de = de.lhs() - de.rhs()
        # consider to add warning if the solution is not unique
        de = solve(de, diff(dvar, ivar), solution_dict=True)
        if len(de) != 1:
            raise NotImplementedError("Sorry, cannot find explicit formula for right-hand side of the ODE.")
        with SR.temp_var() as dummy_dvar:
            return desolve_rk4_inner(de[0][diff(dvar, ivar)].subs({dvar: dummy_dvar}), dummy_dvar)
    else:
        return desolve_rk4_inner(de, dvar)


def desolve_system_rk4(des, vars, ics=None, ivar=None, end_points=None, step=0.1):
    r"""
    Solve numerically a system of first-order ordinary differential
    equations using the `4`-th order Runge-Kutta method. Wrapper for
    Maxima command ``rk``.

    INPUT:

    Input is similar to ``desolve_system`` and ``desolve_rk4`` commands

    - ``des`` -- right hand sides of the system

    - ``vars`` -- dependent variables

    - ``ivar`` -- (optional) should be specified, if there are more variables or
      if the equation is autonomous and the independent variable is
      missing

    - ``ics`` -- initial conditions in the form ``[x0,y01,y02,y03,....]``

    - ``end_points`` -- the end points of the interval

      - if ``end_points`` is a or [a], we integrate on between ``min(ics[0], a)`` and ``max(ics[0], a)``
      - if ``end_points`` is None, we use ``end_points=ics[0]+10``

      - if ``end_points`` is [a,b] we integrate on between ``min(ics[0], a)`` and ``max(ics[0], b)``

    - ``step`` -- (default: 0.1) the length of the step

    OUTPUT: a list of points

    .. SEEALSO::

        :func:`ode_solver`.

    EXAMPLES::

        sage: from sage.calculus.desolvers import desolve_system_rk4

    Lotka Volterra system::

        sage: from sage.calculus.desolvers import desolve_system_rk4
        sage: x,y,t = var('x y t')
        sage: P = desolve_system_rk4([x*(1-y),-y*(1-x)], [x,y], ics=[0,0.5,2],
        ....:                        ivar=t, end_points=20)
        sage: Q = [[i,j] for i,j,k in P]
        sage: LP = list_plot(Q)                                                         # needs sage.plot

        sage: Q = [[j,k] for i,j,k in P]
        sage: LP = list_plot(Q)                                                         # needs sage.plot

    ALGORITHM:

    `4`-th order Runge-Kutta method. Wrapper for command ``rk`` in Maxima's
    dynamics package.  Perhaps could be faster by using ``fast_float``
    instead.

    AUTHOR:

    - Robert Marik (10-2009)
    """
    if ics is None:
        raise ValueError("No initial conditions, specify with ics=[x0,y01,y02,...].")

    ivars = set()

    for de in des:
        ivars = ivars.union(set(de.variables()))
    if ivar is None:
        ivars = ivars - set(vars)
        if len(ivars) != 1:
            raise ValueError("Unable to determine independent variable, please specify.")
        ivar = list(ivars)[0]

    dess = [de._maxima_().str() for de in des]
    desstr = "[" + ",".join(dess) + "]"
    varss = [varsi._maxima_().str() for varsi in vars]
    varstr = "[" + ",".join(varss) + "]"
    x0 = ics[0]
    icss = [ics[i]._maxima_().str() for i in range(1, len(ics))]
    icstr = "[" + ",".join(icss) + "]"
    step = abs(step)

    maxima("load('dynamics)")
    lower_bound, upper_bound = desolve_rk4_determine_bounds(ics, end_points)
    sol_1, sol_2 = [], []
    if lower_bound < ics[0]:
        cmd = "rk(%s,%s,%s,[%s,%s,%s,%s])\
        " % (desstr, varstr, icstr, '_SAGE_VAR_' + str(ivar), str(x0), lower_bound, -step)
        sol_1 = maxima(cmd).sage()
        sol_1.pop(0)
        sol_1.reverse()
    if upper_bound > ics[0]:
        cmd = "rk(%s,%s,%s,[%s,%s,%s,%s])\
        " % (desstr, varstr, icstr, '_SAGE_VAR_' + str(ivar), str(x0), upper_bound, step)
        sol_2 = maxima(cmd).sage()
        sol_2.pop(0)
    sol = sol_1
    sol.append(ics)
    sol.extend(sol_2)

    return sol


def desolve_odeint(des, ics, times, dvars, ivar=None, compute_jac=False, args=(),
                   rtol=None, atol=None, tcrit=None, h0=0.0, hmax=0.0, hmin=0.0, ixpr=0,
                   mxstep=0, mxhnil=0, mxordn=12, mxords=5, printmessg=0):
    r"""
    Solve numerically a system of first-order ordinary differential equations
    using :func:`scipy:scipy.integrate.odeint`.

    INPUT:

    - ``des`` -- right hand sides of the system

    - ``ics`` -- initial conditions

    - ``times`` -- a sequence of time points in which the solution must be found

    - ``dvars`` -- dependent variables. ATTENTION: the order must be the same as
      in ``des``, that means: ``d(dvars[i])/dt=des[i]``

    - ``ivar`` -- independent variable, optional

    - ``compute_jac`` -- boolean (default: ``False``); if ``True``, the
      Jacobian of ``des`` is computed and used during the integration of stiff
      systems

    Other Parameters (taken from the documentation of the
    :func:`~scipy:scipy.integrate.odeint` function from
    :mod:`scipy:scipy.integrate`):

    - ``rtol``, ``atol`` : float
      The input parameters ``rtol`` and ``atol`` determine the error
      control performed by the solver.  The solver will control the
      vector, `e`, of estimated local errors in `y`, according to an
      inequality of the form:

        max-norm of (e / ewt) <= 1

      where ewt is a vector of positive error weights computed as:

        ewt = rtol * abs(y) + atol

      ``rtol`` and ``atol`` can be either vectors the same length as `y` or scalars.

    - ``tcrit`` : array
      Vector of critical points (e.g. singularities) where integration
      care should be taken.

    - ``h0`` : float, (0: solver-determined)
      The step size to be attempted on the first step.

    - ``hmax`` : float, (0: solver-determined)
      The maximum absolute step size allowed.

    - ``hmin`` : float, (0: solver-determined)
      The minimum absolute step size allowed.

    - ``ixpr`` : boolean.
      Whether to generate extra printing at method switches.

    - ``mxstep`` : integer, (0: solver-determined)
      Maximum number of (internally defined) steps allowed for each
      integration point in t.

    - ``mxhnil`` : integer, (0: solver-determined)
      Maximum number of messages printed.

    - ``mxordn`` : integer, (0: solver-determined)
      Maximum order to be allowed for the nonstiff (Adams) method.

    - ``mxords`` : integer, (0: solver-determined)
      Maximum order to be allowed for the stiff (BDF) method.

    OUTPUT: a list with the solution of the system at each time in ``times``

    EXAMPLES:

    Lotka Volterra Equations::

        sage: from sage.calculus.desolvers import desolve_odeint
        sage: x,y = var('x,y')
        sage: f = [x*(1-y), -y*(1-x)]
        sage: sol = desolve_odeint(f, [0.5,2], srange(0,10,0.1), [x,y])                 # needs scipy
        sage: p = line(zip(sol[:,0],sol[:,1]))                                          # needs scipy sage.plot
        sage: p.show()                                                                  # needs scipy sage.plot

    Lorenz Equations::

        sage: x,y,z = var('x,y,z')
        sage: # Next we define the parameters
        sage: sigma = 10
        sage: rho = 28
        sage: beta = 8/3
        sage: # The Lorenz equations
        sage: lorenz = [sigma*(y-x),x*(rho-z)-y,x*y-beta*z]
        sage: # Time and initial conditions
        sage: times = srange(0,50.05,0.05)
        sage: ics = [0,1,1]
        sage: sol = desolve_odeint(lorenz, ics, times, [x,y,z],                         # needs scipy
        ....:                      rtol=1e-13, atol=1e-14)

    One-dimensional stiff system::

        sage: y = var('y')
        sage: epsilon = 0.01
        sage: f = y^2*(1-y)
        sage: ic = epsilon
        sage: t = srange(0,2/epsilon,1)
        sage: sol = desolve_odeint(f, ic, t, y,                                         # needs scipy
        ....:                      rtol=1e-9, atol=1e-10, compute_jac=True)
        sage: p = points(zip(t,sol[:,0]))                                               # needs scipy sage.plot
        sage: p.show()                                                                  # needs scipy sage.plot

    Another stiff system with some optional parameters with no
    default value::

        sage: y1,y2,y3 = var('y1,y2,y3')
        sage: f1 = 77.27*(y2+y1*(1-8.375*1e-6*y1-y2))
        sage: f2 = 1/77.27*(y3-(1+y1)*y2)
        sage: f3 = 0.16*(y1-y3)
        sage: f = [f1,f2,f3]
        sage: ci = [0.2,0.4,0.7]
        sage: t = srange(0,10,0.01)
        sage: v = [y1,y2,y3]
        sage: sol = desolve_odeint(f, ci, t, v, rtol=1e-3, atol=1e-4,                   # needs scipy
        ....:                      h0=0.1, hmax=1, hmin=1e-4, mxstep=1000, mxords=17)

    AUTHOR:

    - Oriol Castejon (05-2010)
    """

    from scipy.integrate import odeint

    from sage.calculus.functions import jacobian
    from sage.ext.fast_eval import fast_float

    def desolve_odeint_inner(ivar):
        # one-dimensional systems:
        if len(dvars) == 1:
            assert len(des) == 1
            dvar = dvars[0]
            de = des[0]
            func = lambda y, t: fast_float(de, dvar, ivar)(y.item(), t)
            if not compute_jac:
                Dfun = None
            else:
                J = diff(de, dvar)
                J = fast_float(J, dvar, ivar)

                def Dfun(y, t):
                    return [J(y.item(), t)]

        # n-dimensional systems:
        else:
            variabs = dvars[:]
            variabs.append(ivar)
            desc = [fast_float(de, *variabs) for de in des]

            def func(y, t):
                v = list(y[:])
                v.append(t)
                return [dec(*v) for dec in desc]

            if not compute_jac:
                Dfun = None
            else:
                J = jacobian(des, dvars)
                J = [list(v) for v in J]
                J = fast_float(J, *variabs)

                def Dfun(y, t):
                    v = list(y[:])
                    v.append(t)
                    return [[element(*v) for element in row] for row in J]

        return odeint(func, ics, times, args=args, Dfun=Dfun, rtol=rtol, atol=atol,
                      tcrit=tcrit, h0=h0, hmax=hmax, hmin=hmin, ixpr=ixpr, mxstep=mxstep,
                      mxhnil=mxhnil, mxordn=mxordn, mxords=mxords, printmessg=printmessg)

    if isinstance(dvars, Expression) and dvars.is_symbol():
        dvars = [dvars]

    if not isinstance(des, (list, tuple)):
        des = [des]

    if ivar is None:
        all_vars = set().union(*[de.variables() for de in des])
        ivars = all_vars - set(dvars)

        if len(ivars) == 1:
            return desolve_odeint_inner(next(iter(ivars)))
        if not ivars:
            from sage.symbolic.ring import SR
            with SR.temp_var() as ivar:
                return desolve_odeint_inner(ivar)
        else:
            raise ValueError("Unable to determine independent variable, please specify.")
    return desolve_odeint_inner(ivar)


def desolve_mintides(f, ics, initial, final, delta, tolrel=1e-16, tolabs=1e-16):
    r"""
    Solve numerically a system of first order differential equations using the
    taylor series integrator implemented in mintides.

    INPUT:

    - ``f`` -- symbolic function; its first argument will be the independent
      variable, . Its output should be de derivatives of the dependent variables.

    - ``ics`` -- list or tuple with the initial conditions

    - ``initial`` -- the starting value for the independent variable

    - ``final`` -- the final value for the independent value

    - ``delta`` -- the size of the steps in the output

    - ``tolrel`` -- the relative tolerance for the method

    - ``tolabs`` -- the absolute tolerance for the method

    OUTPUT: list with the positions of the IVP

    EXAMPLES:

    We integrate a periodic orbit of the Kepler problem along 50 periods::

        sage: var('t,x,y,X,Y')
        (t, x, y, X, Y)
        sage: f(t,x,y,X,Y)=[X, Y, -x/(x^2+y^2)^(3/2), -y/(x^2+y^2)^(3/2)]
        sage: ics = [0.8, 0, 0, 1.22474487139159]
        sage: t = 100*pi
        sage: sol = desolve_mintides(f, ics, 0, t, t, 1e-12, 1e-12) # optional -tides
        sage: sol # optional -tides # abs tol 1e-5
        [[0.000000000000000,
        0.800000000000000,
        0.000000000000000,
        0.000000000000000,
        1.22474487139159],
        [314.159265358979,
        0.800000000028622,
        -5.91973525754241e-9,
        7.56887091890590e-9,
        1.22474487136329]]


    ALGORITHM:

    Uses TIDES.

    REFERENCES:

    - A. Abad, R. Barrio, F. Blesa, M. Rodriguez. Algorithm 924. *ACM
      Transactions on Mathematical Software* , *39* (1), 1-28.

    - A. Abad, R. Barrio, F. Blesa, M. Rodriguez.
      `TIDES tutorial: Integrating ODEs by using the Taylor Series Method.
      <https://web.archive.org/web/20120206041615/http://www.unizar.es/acz/05Publicaciones/Monografias/MonografiasPublicadas/Monografia36/IndMonogr36.htm>`_
    """
    import subprocess
    if subprocess.call('command -v gcc', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE):
        raise RuntimeError('Unable to run because gcc cannot be found')
    from sage.interfaces.tides import genfiles_mintides
    from sage.misc.temporary_file import tmp_dir
    tempdir = tmp_dir()
    intfile = os.path.join(tempdir, 'integrator.c')
    drfile = os.path.join(tempdir, 'driver.c')
    fileoutput = os.path.join(tempdir, 'output')
    runmefile = os.path.join(tempdir, 'runme')
    genfiles_mintides(intfile, drfile, f, [N(_) for _ in ics],
                      N(initial), N(final), N(delta), N(tolrel),
                      N(tolabs), fileoutput)
    subprocess.check_call('gcc -o ' + runmefile + ' ' + os.path.join(tempdir, '*.c ') +
                          os.path.join('$SAGE_LOCAL', 'lib', 'libTIDES.a') + ' $LDFLAGS '
                          + os.path.join('-L$SAGE_LOCAL', 'lib ') + ' -lm  -O2 ' +
                          os.path.join('-I$SAGE_LOCAL', 'include '),
                          shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    subprocess.check_call(os.path.join(tempdir, 'runme'), shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    with open(fileoutput) as outfile:
        res = outfile.readlines()
    for i in range(len(res)):
        res[i] = [RealField()(_) for _ in res[i].split(' ') if len(_) > 2]
    shutil.rmtree(tempdir)
    return res


def desolve_tides_mpfr(f, ics, initial, final, delta, tolrel=1e-16, tolabs=1e-16, digits=50):
    r"""
    Solve numerically a system of first order differential equations using the
    taylor series integrator in arbitrary precision implemented in tides.

    INPUT:

    - ``f`` -- symbolic function; its first argument will be the independent
      variable. Its output should be de derivatives of the dependent variables.

    - ``ics`` -- list or tuple with the initial conditions

    - ``initial`` -- the starting value for the independent variable

    - ``final`` -- the final value for the independent value

    - ``delta`` -- the size of the steps in the output

    - ``tolrel`` -- the relative tolerance for the method

    - ``tolabs`` -- the absolute tolerance for the method

    - ``digits`` -- the digits of precision used in the computation

    OUTPUT: list with the positions of the IVP

    EXAMPLES:

    We integrate the Lorenz equations with Saltzman values for the parameters
    along 10 periodic orbits with 100 digits of precision::

        sage: var('t,x,y,z')
        (t, x, y, z)
        sage: s = 10
        sage: r = 28
        sage: b = 8/3
        sage: f(t,x,y,z)= [s*(y-x),x*(r-z)-y,x*y-b*z]
        sage: x0 = -13.7636106821342005250144010543616538641008648540923684535378642921202827747268115852940239346395038284
        sage: y0 = -19.5787519424517955388380414460095588661142400534276438649791334295426354746147526415973165506704676171
        sage: z0 = 27
        sage: T = 15.586522107161747275678702092126960705284805489972439358895215783190198756258880854355851082660142374
        sage: sol = desolve_tides_mpfr(f, [x0, y0, z0], 0, T, T, 1e-100, 1e-100, 100) # optional - tides
        sage: sol # optional -tides # abs tol 1e-50
        [[0.000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000,
        -13.7636106821342005250144010543616538641008648540923684535378642921202827747268115852940239346395038,
        -19.5787519424517955388380414460095588661142400534276438649791334295426354746147526415973165506704676,
        27.0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000],
        [15.5865221071617472756787020921269607052848054899724393588952157831901987562588808543558510826601424,
        -13.7636106821342005250144010543616538641008648540923684535378642921202827747268115852940239346315658,
        -19.5787519424517955388380414460095588661142400534276438649791334295426354746147526415973165506778440,
        26.9999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999636628]]


    ALGORITHM:

    Uses TIDES.


    .. WARNING::

        This requires the package tides.

    REFERENCES:

    - [ABBR2011]_
    - [ABBR2012]_
    """
    import subprocess
    if subprocess.call('command -v gcc', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE):
        raise RuntimeError('Unable to run because gcc cannot be found')
    from sage.functions.log import log
    from sage.functions.other import ceil
    from sage.interfaces.tides import genfiles_mpfr
    from sage.misc.temporary_file import tmp_dir
    tempdir = tmp_dir()
    intfile = os.path.join(tempdir, 'integrator.c')
    drfile = os.path.join(tempdir, 'driver.c')
    fileoutput = os.path.join(tempdir, 'output')
    runmefile = os.path.join(tempdir, 'runme')
    genfiles_mpfr(intfile, drfile, f, ics, initial, final, delta, [], [],
                  digits, tolrel, tolabs, fileoutput)
    subprocess.check_call('gcc -o ' + runmefile + ' ' + os.path.join(tempdir, '*.c ') +
                          os.path.join('$SAGE_LOCAL', 'lib', 'libTIDES.a') + ' $LDFLAGS '
                          + os.path.join('-L$SAGE_LOCAL', 'lib ') + '-lmpfr -lgmp -lm  -O2 -w ' +
                          os.path.join('-I$SAGE_LOCAL', 'include '),
                          shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    subprocess.check_call(os.path.join(tempdir, 'runme'), shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    with open(fileoutput) as outfile:
        res = outfile.readlines()
    for i in range(len(res)):
        res[i] = [RealField(ceil(digits * log(10, 2)))(piece)
                  for piece in res[i].split(' ') if len(piece) > 2]
    shutil.rmtree(tempdir)
    return res



"""
Root system data for (untwisted) type G affine
"""
#*****************************************************************************
#       Copyright (C) 2008-2009 Daniel Bump
#       Copyright (C) 2008-2009 Justin Walker
#       Copyright (C) 2008-2009 Nicolas M. Thiery <nthiery at users.sf.net>,
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from .cartan_type import CartanType_standard_untwisted_affine


class CartanType(CartanType_standard_untwisted_affine):
    def __init__(self):
        """
        EXAMPLES::

            sage: ct = CartanType(['G',2,1])
            sage: ct
            ['G', 2, 1]
            sage: ct._repr_(compact = True)
            'G2~'

            sage: ct.is_irreducible()
            True
            sage: ct.is_finite()
            False
            sage: ct.is_affine()
            True
            sage: ct.is_untwisted_affine()
            True
            sage: ct.is_crystallographic()
            True
            sage: ct.is_simply_laced()
            False
            sage: ct.classical()
            ['G', 2]
            sage: ct.dual()
            ['G', 2, 1]^*
            sage: ct.dual().is_untwisted_affine()
            False

        TESTS::

            sage: TestSuite(ct).run()
        """
        CartanType_standard_untwisted_affine.__init__(self, "G",2)

    def dynkin_diagram(self):
        """
        Return the extended Dynkin diagram for type G.

        EXAMPLES::

            sage: g = CartanType(['G',2,1]).dynkin_diagram(); g                         # needs sage.graphs
              3
            O=<=O---O
            1   2   0
            G2~
            sage: g.edges(sort=True)                                                    # needs sage.graphs
            [(0, 2, 1), (1, 2, 1), (2, 0, 1), (2, 1, 3)]
        """
        from .dynkin_diagram import DynkinDiagram_class
        g = DynkinDiagram_class(self)
        g.add_edge(1, 2)
        g.set_edge_label(2,1,3)
        g.add_edge(0, 2)
        return g

    def _latex_dynkin_diagram(self, label=None, node=None, node_dist=2, dual=False):
        r"""
        Return a latex representation of the Dynkin diagram.

        EXAMPLES::

            sage: print(CartanType(['G',2,1])._latex_dynkin_diagram())
            \draw (2 cm,0) -- (4.0 cm,0);
            \draw (0, 0.15 cm) -- +(2 cm,0);
            \draw (0, -0.15 cm) -- +(2 cm,0);
            \draw (0,0) -- (2 cm,0);
            \draw (0, 0.15 cm) -- +(2 cm,0);
            \draw (0, -0.15 cm) -- +(2 cm,0);
            \draw[shift={(0.8, 0)}, rotate=180] (135 : 0.45cm) -- (0,0) -- (-135 : 0.45cm);
            \draw[fill=white] (0 cm, 0 cm) circle (.25cm) node[below=4pt]{$1$};
            \draw[fill=white] (2 cm, 0 cm) circle (.25cm) node[below=4pt]{$2$};
            \draw[fill=white] (4 cm, 0 cm) circle (.25cm) node[below=4pt]{$0$};
            <BLANKLINE>
        """
        if label is None:
            label = lambda x: x
        if node is None:
            node = self._latex_draw_node
        ret = "\\draw (%s cm,0) -- (%s cm,0);\n" % (node_dist, node_dist*2.0)
        ret += "\\draw (0, 0.15 cm) -- +(%s cm,0);\n" % node_dist
        ret += "\\draw (0, -0.15 cm) -- +(%s cm,0);\n" % node_dist
        ret += self.classical()._latex_dynkin_diagram(label, node, node_dist, dual)
        ret += node(2*node_dist, 0, label(0))
        return ret

    def ascii_art(self, label=None, node=None):
        """
        Return an ascii art representation of the Dynkin diagram.

        EXAMPLES::

            sage: print(CartanType(['G',2,1]).ascii_art(label = lambda x: x+2))
              3
            O=<=O---O
            3   4   2
        """
        if label is None:
            label = lambda i: i
        if node is None:
            node = self._ascii_art_node
        ret = "  3\n{}=<={}---{}".format(node(label(1)), node(label(2)), node(label(0)))
        return ret + "\n{!s:4}{!s:4}{!s:4}".format(label(1), label(2), label(0))

    def _default_folded_cartan_type(self):
        """
        Return the default folded Cartan type.

        EXAMPLES::

            sage: CartanType(['G', 2, 1])._default_folded_cartan_type()
            ['G', 2, 1] as a folding of ['D', 4, 1]
        """
        from sage.combinat.root_system.type_folded import CartanTypeFolded
        return CartanTypeFolded(self, ['D', 4, 1], [[0], [1, 3, 4], [2]])



"""
Quiver Paths
"""
# ****************************************************************************
#  Copyright (C) 2012    Jim Stark <jstarx@gmail.com>
#                2013/14 Simon King <simon.king@uni-jena.de>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty
#    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
#  See the GNU General Public License for more details; the full text
#  is available at:
#
#                  https://www.gnu.org/licenses/
# ****************************************************************************

cimport cython
from cysignals.signals cimport sig_check, sig_on, sig_off

from sage.data_structures.bounded_integer_sequences cimport *
from cpython.slice cimport PySlice_GetIndicesEx
from sage.structure.richcmp cimport rich_to_bool
from sage.data_structures.bitset_base cimport *


cdef class QuiverPath(MonoidElement):
    r"""
    Class for paths in a quiver.

    A path is given by two vertices, ``start`` and ``end``, and a finite
    (possibly empty) list of edges `e_1, e_2, \ldots, e_n` such that the
    initial vertex of `e_1` is ``start``, the final vertex of `e_i` is
    the initial vertex of `e_{i+1}`, and the final vertex of `e_n` is
    ``end``.  In the case where no edges are specified, we must have
    ``start = end`` and the path is called the trivial path at the given
    vertex.

    .. NOTE::

        Do *not* use this constructor directly! Instead, pass the input to the
        path semigroup that shall be the parent of this path.

    EXAMPLES:

    Specify a path by giving a list of edges::

        sage: Q = DiGraph({1:{2:['a','d'], 3:['e']}, 2:{3:['b']}, 3:{1:['f'], 4:['c']}})
        sage: F = Q.path_semigroup()
        sage: p = F([(1, 2, 'a'), (2, 3, 'b')])
        sage: p
        a*b

    Paths are not *unique*, but different representations of "the same" path
    yield *equal* paths::

        sage: q = F([(1, 1)]) * F([(1, 2, 'a'), (2, 3, 'b')]) * F([(3, 3)])
        sage: p is q
        False
        sage: p == q
        True

    The ``*`` operator is concatenation of paths. If the two paths do not
    compose, its result is ``None``::

        sage: print(p*q)
        None
        sage: p*F([(3, 4, 'c')])
        a*b*c
        sage: F([(2,3,'b'), (3,1,'f')])*p
        b*f*a*b

    The length of a path is the number of edges in that path.  Trivial paths
    are therefore length-`0`::

        sage: len(p)
        2
        sage: triv = F([(1, 1)])
        sage: len(triv)
        0

    List index and slice notation can be used to access the edges in a path.
    QuiverPaths can also be iterated over.  Trivial paths have no elements::

        sage: for x in p: print(x)
        (1, 2, 'a')
        (2, 3, 'b')
        sage: list(triv)
        []

    There are methods giving the initial and terminal vertex of a path::

        sage: p.initial_vertex()
        1
        sage: p.terminal_vertex()
        3
    """
    def __dealloc__(self):
        """
        TESTS::

            sage: from sage.quivers.paths import QuiverPath
            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: p = Q([(1, 1)]) * Q([(1, 1)])
            sage: del p    # indirect doctest
        """
        biseq_dealloc(self._path)

    cdef QuiverPath _new_(self, int start, int end):
        """
        TESTS::

            sage: from sage.quivers.paths import QuiverPath
            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: p = Q(['a']) * Q(['b'])    # indirect doctest
        """
        cdef QuiverPath out = QuiverPath.__new__(self._parent.element_class)
        out._parent = self._parent
        out._start = start
        out._end = end
        return out

    def __init__(self, parent, start, end, path):
        """
        Create a path object.  Type ``QuiverPath?`` for more information.

        INPUT:

        - ``parent`` -- a path semigroup
        - ``start`` -- integer; the label of the initial vertex
        - ``end`` -- integer; the label of the terminal vertex
        - ``path`` -- list of integers, providing the list of arrows
          occurring in the path, labelled according to the position in
          the list of all arrows (resp. the list of outgoing arrows at
          each vertex)

        TESTS::

            sage: from sage.quivers.paths import QuiverPath
            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: p = Q([(1, 1)]) * Q([(1, 1)])
            sage: Q([(1,3,'x')])
            Traceback (most recent call last):
            ...
            ValueError: (1, 3, 'x') is not an edge

        Note that QuiverPath should not be called directly, because
        the elements of the path semigroup associated with a quiver
        may use a sub-class of QuiverPath. Nonetheless, just for test, we
        show that it *is* possible to create a path in a deprecated way::

            sage: p == QuiverPath(Q, 1, 1, [])
            True
            sage: list(Q([(1, 1)])*Q([(1, 2, 'a')])*Q([(2, 2)])*Q([(2, 3, 'b')])*Q([(3, 3)]))
            [(1, 2, 'a'), (2, 3, 'b')]
        """
        MonoidElement.__init__(self, parent=parent)
        self._start = start
        self._end = end
        biseq_init_list(self._path, path, parent._nb_arrows)

    def __reduce__(self):
        """
        TESTS::

            sage: from sage.quivers.paths import QuiverPath
            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: p = Q(['a']) * Q(['b'])
            sage: loads(dumps(p)) == p   # indirect doctest
            True
            sage: loads(dumps(p)) is p
            False
        """
        return NewQuiverPath, (self._parent, self._start, self._end,
                               biseq_pickle(self._path))

    def __hash__(self):
        """
        TESTS::

            sage: from sage.quivers.paths import QuiverPath
            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: p = Q(['a']) * Q(['b'])
            sage: q = Q([(1, 1)])
            sage: {p:1, q:2}[Q(['a','b'])]    # indirect doctest
            1
        """
        if self._path.length == 0:
            return hash(self._start)
        cdef Py_hash_t h = self._start*(<Py_hash_t>1073807360) + biseq_hash(self._path)
        if h == -1:
            return -2
        return h
        # bitset_hash is not a good hash either
        # We should consider using FNV-1a hash, see http://www.isthe.com/chongo/tech/comp/fnv/,
        # Or the hash defined in http://burtleburtle.net/bob/hash/doobs.html
        # Or http://www.azillionmonkeys.com/qed/hash.html

    def _repr_(self):
        r"""
        Default representation of a path.

        TESTS::

            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: Q([(1, 2, 'a'), (2, 3, 'b')]) # indirect doctest
            a*b
            sage: Q([(1, 1)]) # indirect doctest
            e_1
        """
        cdef mp_size_t i
        if not self._path.length:
            return 'e_{0}'.format(self._start)
        L = self._parent._labels
        return '*'.join(L[biseq_getitem(self._path, i)]
                        for i in range(self._path.length))

    def __len__(self):
        """
        Return the length of the path.

        ``length()`` and ``degree()`` are aliases

        TESTS::

            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: len(Q([(1, 2, 'a'), (2, 3, 'b')]))
            2
            sage: Q([(1, 1)]).degree()
            0
            sage: Q([(1, 2, 'a')]).length()
            1
        """
        return self._path.length

    degree = __len__
    length = __len__

    def __bool__(self):
        """
        Implement boolean values for paths.

        .. NOTE::

            The boolean value is ``True`` if and only if this path is of
            positive length.

        TESTS::

            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: a = Q([(1, 2, 'a')])
            sage: b = Q([(2, 3, 'b')])
            sage: bool(a*b)
            True
            sage: bool(Q.idempotents()[0])
            False
        """
        return self._path.length != 0

    cpdef _richcmp_(left, right, int op):
        """
        Comparison for :class:`QuiverPaths`.

        The following data (listed in order of preference) is used for
        comparison:

        - **Negative** length of the paths
        - initial and terminal vertices of the paths
        - Edge sequence of the paths, by reverse lexicographical ordering.

        .. NOTE::

            This code is used by :class:`CombinatorialFreeModule` to order
            the monomials when printing elements of path algebras.

        EXAMPLES:

        A path semigroup of a quiver with a single vertex has a multiplicative
        unit::

            sage: D = DiGraph({0:{0:['x','y','z']}}).path_semigroup()
            sage: D(1)
            e_0
            sage: D in Monoids()
            True

        However, there is no coercion from the ring of integers and hence the
        generic comparison code finds that the multiplicative units in `D` and
        in the ring of integers evaluate unequal::

            sage: D.has_coerce_map_from(ZZ)
            False
            sage: D(1) == 1
            False

        TESTS::

            sage: Q = DiGraph({1:{2:['a', 'b']}, 2:{3:['c'], 4:['d']}}).path_semigroup()
            sage: a = Q([(1, 2, 'a')])
            sage: b = Q([(1, 2, 'b')])
            sage: c = Q([(2, 3, 'c')])
            sage: d = Q([(2, 4, 'd')])
            sage: e = Q.idempotents()[3]
            sage: e < a  # e is shorter than a
            False
            sage: a < e
            True
            sage: d < a*c
            False
            sage: a*c < d
            True
            sage: a < b
            True
            sage: b < a
            False
            sage: a*c < a*d
            True
            sage: a*d < a*c
            False
            sage: a < a
            False
        """
        # Since QuiverPath inherits from Element, it is guaranteed that
        # both arguments are elements of the same path semigroup
        cdef QuiverPath cself, other
        cself = left
        other = right
        # we want *negative* degree reverse lexicographical order
        if other._path.length < cself._path.length:
            return rich_to_bool(op, -1)
        if other._path.length > cself._path.length:
            return rich_to_bool(op, 1)
        if cself._start < other._start:
            return rich_to_bool(op, -1)
        if cself._start > other._start:
            return rich_to_bool(op, 1)
        if cself._end < other._end:
            return rich_to_bool(op, -1)
        if cself._end > other._end:
            return rich_to_bool(op, 1)
        if cself._path.length == 0:
            return rich_to_bool(op, 0)
        return biseq_richcmp(cself._path, other._path, op)

    def __getitem__(self, index):
        """
        Implement index notation.

        TESTS::

            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}, 3:{4:['c'], 1:['d']}}).path_semigroup()
            sage: p = Q([(1, 2, 'a'), (2, 3, 'b'), (3, 1, 'd'), (1, 2, 'a'), (2, 3, 'b'), (3, 4, 'c')])
            sage: p
            a*b*d*a*b*c

        A single index returns the arrow that appears in the path at this index::

            sage: p[0]
            a
            sage: p[-1]
            c

        A slice with step 1 returns a sub-path of this path::

            sage: p[1:5]
            b*d*a*b

        A slice with step -1 return a sub-path of the reversed path::

            sage: p[4:1:-1]
            b*a*d

        If the start index is greater than the terminal index and the step
        -1 is not explicitly given, then a path of length zero is returned,
        which is compatible with Python lists::

            sage: list(range(6))[4:1]
            []

        The following was fixed in :issue:`22278`. A path slice of length
        zero of course has a specific start- and endpoint. It is always
        the startpoint of the arrow corresponding to the first item of
        the range::

            sage: p[4:1]
            e_2
            sage: p[4:1].initial_vertex() == p[4].initial_vertex()
            True

        If the slice boundaries are out of bound, then no error is raised,
        which is compatible with Python lists::

            sage: list(range(6))[20:40]
            []

        In that case, the startpoint of the slice of length zero is the
        endpoint of the path::

            sage: p[20:40]
            e_4
            sage: p[20:40].initial_vertex() == p.terminal_vertex()
            True
        """
        cdef tuple E
        cdef Py_ssize_t start, stop, step, slicelength
        cdef int init, end
        cdef QuiverPath OUT
        if isinstance(index, slice):
            PySlice_GetIndicesEx(index, self._path.length,
                                 &start, &stop, &step,
                                 &slicelength)
            if step != 1 and step != -1:
                raise ValueError("slicing only possible for step +/-1")
            if step == -1:
                return self.reversal()[self._path.length-1-start:self._path.length-1-stop]
            if start == 0 and stop == self._path.length:
                return self
            if start > stop:
                stop=start
            E = self._parent._sorted_edges
            if start < self._path.length:
                init = E[biseq_getitem(self._path, start)][0]
            else:
                init = self._end
            if start < stop:
                end = E[biseq_getitem(self._path, stop-1)][1]
            else:  # the result will be a path of length 0
                end = init
            OUT = self._new_(init, end)
            biseq_init_slice(OUT._path, self._path, start, stop, step)
            return OUT
        if index < 0:
            index = self._path.length+index
        if index < 0 or index >= self._path.length:
            raise IndexError("list index out of range")
        E = self._parent._sorted_edges
        init = E[biseq_getitem(self._path, index)][0]
        end = E[biseq_getitem(self._path, index)][1]
        OUT = self._new_(init, end)
        biseq_init_slice(OUT._path, self._path, index, index+1, 1)
        return OUT

    def __iter__(self):
        """
        Iteration over the path.

        TESTS::

            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}, 3:{4:['c']}}).path_semigroup()
            sage: p = Q([(1, 2, 'a'), (2, 3, 'b'), (3, 4, 'c')])
            sage: for e in p: print(e)
            (1, 2, 'a')
            (2, 3, 'b')
            (3, 4, 'c')
        """
        # Return an iterator over an empty tuple for trivial paths, otherwise
        # return an iterator for _path as a list
        cdef mp_size_t i
        cdef tuple E = self._parent._sorted_edges
        for i in range(self._path.length):
            yield E[biseq_getitem(self._path, i)]

    cpdef _mul_(self, other):
        """
        Compose two paths.

        .. NOTE::

            ``None`` is returned if the terminal vertex of the first path
            does not coincide with the initial vertex of the second path.

        TESTS::

            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}, 3:{4:['c']}, 4:{5:['d']}}).path_semigroup()
            sage: x = Q([(1, 2, 'a'), (2, 3, 'b')])
            sage: y = Q([(3, 4, 'c'), (4, 5, 'd')])
            sage: print(y*x)
            None
            sage: x*y
            a*b*c*d
            sage: x*Q([(3, 4, 'c')])
            a*b*c
            sage: x*Q([(3, 4, 'c'), (4, 5, 'd')])
            a*b*c*d
            sage: x*6
            Traceback (most recent call last):
            ...
            TypeError: unsupported operand parent(s) for *:
             'Partial semigroup formed by the directed paths of Multi-digraph on 5 vertices'
             and 'Integer Ring'
        """
        # By Sage's coercion model, both paths belong to the same quiver
        # In particular, both are QuiverPath
        cdef QuiverPath right = other
        if self._end != right._start:
            return None
        cdef QuiverPath OUT = self._new_(self._start, right._end)
        biseq_init_concat(OUT._path, self._path, right._path)
        return OUT

    cpdef _mod_(self, other):
        """
        Return what remains of this path after removing the initial segment ``other``.

        If ``other`` is not an initial segment of this path then ``None`` is
        returned.  Deleting the trivial path at vertex `v` from a path that
        begins at `v` does not change the path.

        TESTS::

            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: p = Q([(1, 2, 'a'), (2, 3, 'b')])
            sage: a = Q([(1, 2, 'a')])
            sage: b = Q([(2, 3, 'b')])
            sage: e1 = Q([(1, 1)])
            sage: e2 = Q([(2, 2)])
            sage: p % a
            b
            sage: print(p % b)
            None
            sage: p % e1
            a*b
            sage: print(p % e2)
            None
        """
        cdef QuiverPath right = <QuiverPath>other
        # Handle trivial case
        if self._start != right._start:
            return None
        if right._path.length == 0:
            return self

        # If other is the beginning, return the rest
        cdef QuiverPath OUT
        if (self._start == right._start) and biseq_startswith(self._path, right._path):
            OUT = self._new_(right._end, self._end)
            biseq_init_slice(OUT._path, self._path, right._path.length, self._path.length, 1)
            return OUT
        else:
            return None

    def gcd(self, QuiverPath P):
        """
        Greatest common divisor of two quiver paths, with co-factors.

        For paths, by "greatest common divisor", we mean the largest terminal
        segment of the first path that is an initial segment of the second
        path.

        INPUT:

        - ``P`` -- a :class:`QuiverPath`

        OUTPUT:

        - :class:`QuiverPath`s ``(C1,G,C2)`` such that ``self = C1*G`` and ``P = G*C2``, or
        - ``(None, None, None)``, if the paths do not overlap (or belong to different quivers).

        EXAMPLES::

            sage: Q = DiGraph({1:{2:['a']}, 2:{1:['b'], 3:['c']}, 3:{1:['d']}}).path_semigroup()
            sage: p1 = Q(['c','d','a','b','a','c','d'])
            sage: p1
            c*d*a*b*a*c*d
            sage: p2 = Q(['a','b','a','c','d','a','c','d','a','b'])
            sage: p2
            a*b*a*c*d*a*c*d*a*b
            sage: S1, G, S2 = p1.gcd(p2)
            sage: S1, G, S2
            (c*d, a*b*a*c*d, a*c*d*a*b)
            sage: S1*G == p1
            True
            sage: G*S2 == p2
            True
            sage: p2.gcd(p1)
            (a*b*a*c*d*a, c*d*a*b, a*c*d)

        We test that a full overlap is detected::

            sage: p2.gcd(p2)
            (e_1, a*b*a*c*d*a*c*d*a*b, e_1)

        The absence of an overlap is detected::

            sage: p2[2:-1]
            a*c*d*a*c*d*a
            sage: p2[1:]
            b*a*c*d*a*c*d*a*b
            sage: print(p2[2:-1].gcd(p2[1:]))
            (None, None, None)
        """
        if self._parent is not P._parent:
            return (None, None, None)
        cdef size_t i
        sig_on()
        i = biseq_startswith_tail(P._path, self._path, 0)
        sig_off()
        if i == <size_t>-1:
            return (None, None, None)
        return (self[:i], self[i:], P[self._path.length-i:])

    cpdef tuple complement(self, QuiverPath subpath):
        """
        Return a pair ``(a,b)`` of paths s.t. ``self = a*subpath*b``,
        or ``(None, None)`` if ``subpath`` is not a subpath of this path.

        .. NOTE::

            ``a`` is chosen of minimal length.

        EXAMPLES::

            sage: S = DiGraph({1:{1:['a','b','c','d']}}).path_semigroup()
            sage: S.inject_variables()
            Defining e_1, a, b, c, d
            sage: (b*c*a*d*b*a*d*d).complement(a*d)
            (b*c, b*a*d*d)
            sage: (b*c*a*d*b).complement(a*c)
            (None, None)
        """
        cdef mp_size_t i = biseq_contains(self._path, subpath._path, 0)
        if i == -1:
            return (None, None)
        return self[:i], self[i+len(subpath):]

    cpdef bint has_subpath(self, QuiverPath subpath) except -1:
        """
        Tells whether this path contains a given sub-path.

        INPUT:

        - ``subpath`` -- a path of positive length in the same path semigroup
          as this path

        EXAMPLES::

            sage: S = DiGraph({0:{1:['a'], 2:['b']}, 1:{0:['c'], 1:['d']}, 2:{0:['e'],2:['f']}}).path_semigroup()
            sage: S.inject_variables()
            Defining e_0, e_1, e_2, a, b, c, d, e, f
            sage: (c*b*e*a).has_subpath(b*e)
            1
            sage: (c*b*e*a).has_subpath(b*f)
            0
            sage: (c*b*e*a).has_subpath(e_1)
            Traceback (most recent call last):
            ...
            ValueError: we only consider sub-paths of positive length
            sage: (c*b*e*a).has_subpath(None)
            Traceback (most recent call last):
            ...
            ValueError: the given sub-path is empty
        """
        if subpath is None:
            raise ValueError("the given sub-path is empty")
        if subpath._parent is not self._parent:
            raise ValueError("the two paths belong to different quivers")
        if subpath._path.length == 0:
            raise ValueError("we only consider sub-paths of positive length")
        if self._path.length < subpath._path.length:
            return 0
        if biseq_contains(self._path, subpath._path, 0) == -1:
            return 0
        return 1

    cpdef bint has_prefix(self, QuiverPath subpath) except -1:
        """
        Tells whether this path starts with a given sub-path.

        INPUT:

        - ``subpath`` -- a path in the same path semigroup as this path

        OUTPUT: ``0`` or ``1``, which stands for ``False`` resp. ``True``

        EXAMPLES::

            sage: S = DiGraph({0:{1:['a'], 2:['b']}, 1:{0:['c'], 1:['d']}, 2:{0:['e'],2:['f']}}).path_semigroup()
            sage: S.inject_variables()
            Defining e_0, e_1, e_2, a, b, c, d, e, f
            sage: (c*b*e*a).has_prefix(b*e)
            0
            sage: (c*b*e*a).has_prefix(c*b)
            1
            sage: (c*b*e*a).has_prefix(e_1)
            1
            sage: (c*b*e*a).has_prefix(e_2)
            0
        """
        if subpath._parent is not self._parent:
            raise ValueError("the two paths belong to different quivers")
        if self._start != subpath._start:
            return 0
        if subpath._path.length == 0:
            return 1
        if biseq_startswith(self._path, subpath._path):
            return 1
        return 0

    def initial_vertex(self):
        """
        Return the initial vertex of the path.

        OUTPUT: integer; the label of the initial vertex

        EXAMPLES::

            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: y = Q([(1, 2, 'a'), (2, 3, 'b')])
            sage: y.initial_vertex()
            1
        """
        return self._start

    def terminal_vertex(self):
        """
        Return the terminal vertex of the path.

        OUTPUT: integer; the label of the terminal vertex

        EXAMPLES::

            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
            sage: y = Q([(1, 2, 'a'), (2, 3, 'b')])
            sage: y.terminal_vertex()
            3
        """
        return self._end

    def reversal(self):
        """
        Return the path along the same edges in reverse order in the
        opposite quiver.

        EXAMPLES::

            sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}, 3:{4:['c'], 1:['d']}}).path_semigroup()
            sage: p = Q([(1, 2, 'a'), (2, 3, 'b'), (3, 1, 'd'), (1, 2, 'a'), (2, 3, 'b'), (3, 4, 'c')])
            sage: p
            a*b*d*a*b*c
            sage: p.reversal()
            c*b*a*d*b*a
            sage: e = Q.idempotents()[0]
            sage: e
            e_1
            sage: e.reversal()
            e_1
        """
        Q = self._parent.reverse()
        # Handle trivial paths
        if self._path.length == 0:
            return Q.element_class(Q, self._end, self._start, [])

        # Reverse all the edges in the path, then reverse the path
        cdef mp_size_t i
        cdef QuiverPath out = QuiverPath.__new__(Q.element_class)
        out._parent = Q
        out._start = self._end
        out._end = self._start
        sig_check()
        biseq_init(out._path, self._path.length, self._path.itembitsize)
        cdef mp_size_t ell = self._path.length - 1
        for i in range(self._path.length):
            sig_check()
            biseq_inititem(out._path, i, biseq_getitem(self._path, ell - i))
        return out


def NewQuiverPath(Q, start, end, biseq_data):
    """
    Return a new quiver path for given defining data.

    INPUT:

    - ``Q`` -- the path semigroup of a quiver
    - ``start`` -- integer; the label of the startpoint
    - ``end`` -- integer; the label of the endpoint
    - ``biseq_data`` -- tuple formed by

      - a string, encoding a bitmap representing the path as integer
        at base `32`
      - the number of bits used to store the path
      - the number of bits used to store a single item
      - the number of items in the path

    TESTS::

        sage: from sage.quivers.paths import QuiverPath
        sage: Q = DiGraph({1:{2:['a']}, 2:{3:['b']}}).path_semigroup()
        sage: p = Q(['a']) * Q(['b'])
        sage: loads(dumps(p)) == p   # indirect doctest
        True
        sage: p.__reduce__()
        (<cyfunction NewQuiverPath at ...>,
         (Partial semigroup formed by the directed paths of Multi-digraph on 3 vertices,
          1,
          3,
          ((0, 4, 1, ..., (4,)), 2, 2)))
    """
    cdef QuiverPath out = QuiverPath.__new__(Q.element_class)
    out._parent = Q
    out._start = start
    out._end = end
    biseq_unpickle(out._path, biseq_data[0], biseq_data[1], biseq_data[2])
    return out



# sage.doctest: needs sage.combinat sage.modules
r"""
Bases for ``NCSym``

AUTHORS:

- Travis Scrimshaw (08-04-2013): initial version
"""
# ****************************************************************************
#       Copyright (C) 2013 Travis Scrimshaw <tscrim at ucdavis.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.misc.abstract_method import abstract_method
from sage.misc.lazy_attribute import lazy_attribute
from sage.misc.cachefunc import cached_method
from sage.misc.bindable_class import BindableClass
from sage.categories.graded_hopf_algebras import GradedHopfAlgebras
from sage.categories.realizations import Category_realization_of_parent
from sage.categories.modules_with_basis import ModulesWithBasis
from sage.categories.tensor import tensor
from sage.categories.homset import Hom
from sage.combinat.set_partition import SetPartition, SetPartitions
from sage.combinat.free_module import CombinatorialFreeModule


class NCSymBasis_abstract(CombinatorialFreeModule, BindableClass):
    """
    Abstract base class for a basis of `NCSym` or its dual.
    """

    def _element_constructor_(self, x):
        """
        Construct an element of ``self``.

        INPUT:

        - ``x`` -- set partition or list of lists of integers

        EXAMPLES::

            sage: m = SymmetricFunctionsNonCommutingVariables(QQ).m()
            sage: m([[1,3],[2]])
            m{{1, 3}, {2}}
            sage: m(SetPartition([[1,3],[2]]))
            m{{1, 3}, {2}}
        """
        if isinstance(x, (list, tuple)):
            x = SetPartition(x)
        return super()._element_constructor_(x)


class NCSymOrNCSymDualBases(Category_realization_of_parent):
    r"""
    Base category for the category of bases of symmetric functions
    in non-commuting variables or its Hopf dual for the common code.
    """

    def super_categories(self):
        r"""
        Return the super categories of bases of (the Hopf dual of) the
        symmetric functions in non-commuting variables.

        OUTPUT: list of categories

        TESTS::

            sage: from sage.combinat.ncsym.bases import NCSymOrNCSymDualBases
            sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
            sage: NCSymOrNCSymDualBases(NCSym).super_categories()
            [Category of realizations of Symmetric functions in
              non-commuting variables over the Rational Field,
             Category of graded Hopf algebras with basis over Rational Field,
             Join of Category of realizations of Hopf algebras over Rational Field
              and Category of graded algebras over Rational Field
              and Category of graded coalgebras over Rational Field]
        """
        R = self.base().base_ring()
        from sage.categories.graded_hopf_algebras_with_basis import GradedHopfAlgebrasWithBasis
        return [self.base().Realizations(),
                GradedHopfAlgebrasWithBasis(R),
                GradedHopfAlgebras(R).Realizations()]

    def _repr_(self):
        r"""
        Return a string representation of ``self``.

        TESTS::

            sage: from sage.combinat.ncsym.bases import NCSymOrNCSymDualBases
            sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
            sage: NCSymOrNCSymDualBases(NCSym)
            Category of bases of NCSym or NCSym^* over the Rational Field
        """
        return "Category of bases of NCSym or NCSym^* over the {}".format(self.base().base_ring())

    class ParentMethods:
        def _repr_(self):
            """
            TESTS::

                sage: SymmetricFunctionsNonCommutingVariables(QQ).m()
                Symmetric functions in non-commuting variables over the Rational Field in the monomial basis
                sage: SymmetricFunctionsNonCommutingVariables(QQ).m().dual_basis()
                Dual symmetric functions in non-commuting variables over the Rational Field in the w basis
                sage: SymmetricFunctionsNonCommutingVariables(QQ).chi()
                Symmetric functions in non-commuting variables over the Rational Field in the
                 supercharacter basis with parameter q=2
                sage: SymmetricFunctionsNonCommutingVariables(QQ['q'].fraction_field()).rho('q')
                Symmetric functions in non-commuting variables over the Fraction Field
                 of Univariate Polynomial Ring in q over Rational Field in the
                 deformed_coarse_powersum basis with parameter q
            """
            str = "{} in the {} basis".format(self.realization_of(), self._realization_name())
            if hasattr(self, '_q'):
                str += " with parameter q"
                if repr(self._q) != 'q':
                    str += "=" + repr(self._q)
            return str

        def __getitem__(self, i):
            """
            Return the basis element indexed by `i`.

            INPUT:

            - ``i`` -- set partition or a list of list of integers

            EXAMPLES::

                sage: w = SymmetricFunctionsNonCommutingVariables(QQ).dual().w()
                sage: w[[1], [2,3]]
                w{{1}, {2, 3}}
                sage: w[{1}, (2,3)]
                w{{1}, {2, 3}}
                sage: w[[]]
                w{}
            """
            if isinstance(i, SetPartition):
                return self.monomial(i)
            if i == []:
                return self.one()
            if not isinstance(i, tuple):
                i = (i,)
            return self.monomial(SetPartition(i))

        @cached_method
        def one_basis(self):
            r"""
            Return the index of the basis element containing `1`.

            OUTPUT: the empty set partition

            EXAMPLES::

                sage: m = SymmetricFunctionsNonCommutingVariables(QQ).m()
                sage: m.one_basis()
                {}
                sage: w = SymmetricFunctionsNonCommutingVariables(QQ).dual().w()
                sage: w.one_basis()
                {}
            """
            return SetPartition([])

        def counit_on_basis(self, A):
            r"""
            The counit is defined by sending all elements of positive degree
            to zero.

            INPUT:

            - ``A`` -- set partition

            OUTPUT: either the `0` or the `1` of the base ring of ``self``

            EXAMPLES::

                sage: m = SymmetricFunctionsNonCommutingVariables(QQ).m()
                sage: m.counit_on_basis(SetPartition([[1,3], [2]]))
                0
                sage: m.counit_on_basis(SetPartition([]))
                1
                sage: w = SymmetricFunctionsNonCommutingVariables(QQ).dual().w()
                sage: w.counit_on_basis(SetPartition([[1,3], [2]]))
                0
                sage: w.counit_on_basis(SetPartition([]))
                1
            """
            if len(A) != 0:
                return self.base_ring().zero()
            return self.base_ring().one()

        def duality_pairing(self, x, y):
            r"""
            Compute the pairing between an element of ``self`` and an element
            of the dual.

            Carry out this computation by converting ``x`` to the `\mathbf{m}`
            basis and ``y`` to the `\mathbf{w}` basis.

            INPUT:

            - ``x`` -- an element of symmetric functions in non-commuting
              variables
            - ``y`` -- an element of the dual of symmetric functions in
              non-commuting variables

            OUTPUT: an element of the base ring of ``self``

            EXAMPLES::

                sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
                sage: h = NCSym.h()
                sage: w = NCSym.m().dual_basis()
                sage: matrix([[h(A).duality_pairing(w(B)) for A in SetPartitions(3)] for B in SetPartitions(3)])
                [6 2 2 2 1]
                [2 2 1 1 1]
                [2 1 2 1 1]
                [2 1 1 2 1]
                [1 1 1 1 1]
                sage: (h[[1,2],[3]] + 3*h[[1,3],[2]]).duality_pairing(2*w[[1,3],[2]] + w[[1,2,3]] + 2*w[[1,2],[3]])
                32
            """
            m = self.realization_of().m()
            x = m(x)
            y = m.dual_basis()(y)
            return sum(coeff * y[I] for (I, coeff) in x)

        def duality_pairing_matrix(self, basis, degree):
            r"""
            The matrix of scalar products between elements of `NCSym` and
            elements of `NCSym^*`.

            INPUT:

            - ``basis`` -- a basis of the dual Hopf algebra
            - ``degree`` -- nonnegative integer

            OUTPUT:

            The matrix of scalar products between the basis ``self`` and the
            basis ``basis`` in the dual Hopf algebra of degree ``degree``.

            EXAMPLES:

            The matrix between the `\mathbf{m}` basis and the
            `\mathbf{w}` basis::

                sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
                sage: m = NCSym.m()
                sage: w = NCSym.dual().w()
                sage: m.duality_pairing_matrix(w, 3)
                [1 0 0 0 0]
                [0 1 0 0 0]
                [0 0 1 0 0]
                [0 0 0 1 0]
                [0 0 0 0 1]

            Similarly for some of the other basis of `NCSym` and the `\mathbf{w}`
            basis::

                sage: e = NCSym.e()
                sage: e.duality_pairing_matrix(w, 3)
                [0 0 0 0 1]
                [0 0 1 1 1]
                [0 1 0 1 1]
                [0 1 1 0 1]
                [1 1 1 1 1]
                sage: p = NCSym.p()
                sage: p.duality_pairing_matrix(w, 3)
                [1 0 0 0 0]
                [1 1 0 0 0]
                [1 0 1 0 0]
                [1 0 0 1 0]
                [1 1 1 1 1]
                sage: cp = NCSym.cp()
                sage: cp.duality_pairing_matrix(w, 3)
                [1 0 0 0 0]
                [1 1 0 0 0]
                [0 0 1 0 0]
                [1 0 0 1 0]
                [1 1 1 1 1]
                sage: x = NCSym.x()
                sage: w.duality_pairing_matrix(x, 3)
                [ 0  0  0  0  1]
                [ 1  0 -1 -1  1]
                [ 1 -1  0 -1  1]
                [ 1 -1 -1  0  1]
                [ 2 -1 -1 -1  1]

            A base case test::

                sage: m.duality_pairing_matrix(w, 0)
                [1]
            """
            from sage.matrix.constructor import matrix
            # TODO: generalize to keys indexing the basis of the graded component
            return matrix(self.base_ring(),
                    [[self.duality_pairing(self[I], basis[J])
                      for J in SetPartitions(degree)]
                     for I in SetPartitions(degree)])

    class ElementMethods:
        def duality_pairing(self, other):
            r"""
            Compute the pairing between ``self`` and an element ``other`` of the dual.

            EXAMPLES::

                sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
                sage: m = NCSym.m()
                sage: w = m.dual_basis()
                sage: elt = m[[1,3],[2]] - 3*m[[1,2],[3]]
                sage: elt.duality_pairing(w[[1,3],[2]])
                1
                sage: elt.duality_pairing(w[[1,2],[3]])
                -3
                sage: elt.duality_pairing(w[[1,2]])
                0
                sage: e = NCSym.e()
                sage: w[[1,3],[2]].duality_pairing(e[[1,3],[2]])
                0
            """
            return self.parent().duality_pairing(self, other)


class NCSymBases(Category_realization_of_parent):
    r"""
    Category of bases of symmetric functions in non-commuting variables.

    EXAMPLES::

        sage: from sage.combinat.ncsym.bases import NCSymBases
        sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
        sage: NCSymBases(NCSym)
        Category of bases of symmetric functions in non-commuting variables over the Rational Field
    """

    def super_categories(self):
        r"""
        Return the super categories of bases of the Hopf dual of the
        symmetric functions in non-commuting variables.

        OUTPUT: list of categories

        TESTS::

            sage: from sage.combinat.ncsym.bases import NCSymBases
            sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
            sage: NCSymBases(NCSym).super_categories()
            [Category of bases of NCSym or NCSym^* over the Rational Field]
        """
        return [NCSymOrNCSymDualBases(self.base())]

    def _repr_(self):
        r"""
        Return a string representation of ``self``.

        TESTS::

            sage: from sage.combinat.ncsym.bases import NCSymBases
            sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
            sage: NCSymBases(NCSym)
            Category of bases of symmetric functions in non-commuting variables over the Rational Field
        """
        return "Category of bases of symmetric functions in non-commuting"\
               " variables over the {}".format(self.base().base_ring())

    class ParentMethods:
        def from_symmetric_function(self, f):
            """
            Return the image of the symmetric function ``f`` in ``self``.

            This is performed by converting to the monomial basis and
            extending the method :meth:`sum_of_partitions` linearly.  This is a
            linear map from the symmetric functions to the symmetric functions
            in non-commuting variables that does not preserve the product or
            coproduct structure of the Hopf algebra.

            .. SEEALSO:: :meth:`to_symmetric_function`

            INPUT:

            - ``f`` -- a symmetric function

            OUTPUT: an element of ``self``

            EXAMPLES::

                sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
                sage: Sym = SymmetricFunctions(QQ)
                sage: e = NCSym.e()
                sage: elem = Sym.e()
                sage: elt = e.from_symmetric_function(elem[2,1,1]); elt
                1/12*e{{1}, {2}, {3, 4}} + 1/12*e{{1}, {2, 3}, {4}} + 1/12*e{{1}, {2, 4}, {3}}
                 + 1/12*e{{1, 2}, {3}, {4}} + 1/12*e{{1, 3}, {2}, {4}} + 1/12*e{{1, 4}, {2}, {3}}
                sage: elem(elt.to_symmetric_function())
                e[2, 1, 1]
                sage: e.from_symmetric_function(elem[4])
                1/24*e{{1, 2, 3, 4}}
                sage: p = NCSym.p()
                sage: pow = Sym.p()
                sage: elt = p.from_symmetric_function(pow[2,1,1]); elt
                1/6*p{{1}, {2}, {3, 4}} + 1/6*p{{1}, {2, 3}, {4}} + 1/6*p{{1}, {2, 4}, {3}}
                 + 1/6*p{{1, 2}, {3}, {4}} + 1/6*p{{1, 3}, {2}, {4}} + 1/6*p{{1, 4}, {2}, {3}}
                sage: pow(elt.to_symmetric_function())
                p[2, 1, 1]
                sage: p.from_symmetric_function(pow[4])
                p{{1, 2, 3, 4}}
                sage: h = NCSym.h()
                sage: comp = Sym.complete()
                sage: elt = h.from_symmetric_function(comp[2,1,1]); elt
                1/12*h{{1}, {2}, {3, 4}} + 1/12*h{{1}, {2, 3}, {4}} + 1/12*h{{1}, {2, 4}, {3}}
                 + 1/12*h{{1, 2}, {3}, {4}} + 1/12*h{{1, 3}, {2}, {4}} + 1/12*h{{1, 4}, {2}, {3}}
                sage: comp(elt.to_symmetric_function())
                h[2, 1, 1]
                sage: h.from_symmetric_function(comp[4])
                1/24*h{{1, 2, 3, 4}}
            """
            m = self.realization_of().m()
            return self(m.from_symmetric_function(f))

        def primitive(self, A, i=1):
            r"""
            Return the primitive associated to ``A`` in ``self``.

            .. SEEALSO::

                :meth:`~sage.combinat.ncsym.ncsym.SymmetricFunctionsNonCommutingVariables.powersum.primitive`

            INPUT:

            - ``A`` -- set partition
            - ``i`` -- positive integer

            OUTPUT: an element of ``self``

            EXAMPLES::

                sage: e = SymmetricFunctionsNonCommutingVariables(QQ).e()
                sage: elt = e.primitive(SetPartition([[1,3],[2]])); elt
                e{{1, 2}, {3}} - e{{1, 3}, {2}}
                sage: elt.coproduct()
                e{} # e{{1, 2}, {3}} - e{} # e{{1, 3}, {2}} + e{{1, 2}, {3}} # e{} - e{{1, 3}, {2}} # e{}
            """
            p = self.realization_of().p()
            return self(p.primitive(A, i))

        @abstract_method(optional=True)
        def internal_coproduct_on_basis(self, i):
            """
            The internal coproduct of the algebra on the basis (optional).

            INPUT:

            - ``i`` -- the indices of an element of the basis of ``self``

            OUTPUT: an element of the tensor squared of ``self``

            EXAMPLES::

                sage: m = SymmetricFunctionsNonCommutingVariables(QQ).m()
                sage: m.internal_coproduct_on_basis(SetPartition([[1,2]]))
                m{{1, 2}} # m{{1, 2}}
            """

        @lazy_attribute
        def internal_coproduct(self):
            r"""
            Compute the internal coproduct of ``self``.

            If :meth:`internal_coproduct_on_basis()` is available, construct
            the internal coproduct morphism from ``self`` to ``self``
            `\otimes` ``self`` by extending it by linearity. Otherwise, this uses
            :meth:`internal_coproduct_by_coercion()`, if available.

            OUTPUT: an element of the tensor squared of ``self``

            EXAMPLES::

                sage: cp = SymmetricFunctionsNonCommutingVariables(QQ).cp()
                sage: cp.internal_coproduct(cp[[1,3],[2]] - 2*cp[[1]])
                -2*cp{{1}} # cp{{1}} + cp{{1, 2, 3}} # cp{{1, 3}, {2}} + cp{{1, 3}, {2}} # cp{{1, 2, 3}}
                 + cp{{1, 3}, {2}} # cp{{1, 3}, {2}}
            """
            if self.internal_coproduct_on_basis is not NotImplemented:
                return Hom(self, tensor([self, self]),
                           ModulesWithBasis(self.base_ring()))(on_basis=self.internal_coproduct_on_basis)
            if hasattr(self, "internal_coproduct_by_coercion"):
                return self.internal_coproduct_by_coercion

        def internal_coproduct_by_coercion(self, x):
            r"""
            Return the internal coproduct by coercing the element to the powersum basis.

            INPUT:

            - ``x`` -- an element of ``self``

            OUTPUT: an element of the tensor squared of ``self``

            EXAMPLES::

                sage: h = SymmetricFunctionsNonCommutingVariables(QQ).h()
                sage: h[[1,3],[2]].internal_coproduct() # indirect doctest
                2*h{{1}, {2}, {3}} # h{{1}, {2}, {3}} - h{{1}, {2}, {3}} # h{{1, 3}, {2}}
                 - h{{1, 3}, {2}} # h{{1}, {2}, {3}} + h{{1, 3}, {2}} # h{{1, 3}, {2}}
            """
            R = self.realization_of().a_realization()
            return self.tensor_square().sum(coeff * tensor([self(R[A]), self(R[B])])
                                            for ((A, B), coeff) in R(x).internal_coproduct())

    class ElementMethods:
        def expand(self, n, alphabet='x'):
            r"""
            Expand the symmetric function into ``n`` non-commuting
            variables in an alphabet, which by default is ``'x'``.

            This computation is completed by coercing the element ``self``
            into the monomial basis and computing the expansion in
            the ``alphabet`` there.

            INPUT:

            - ``n`` -- the number of variables in the expansion
            - ``alphabet`` -- (default: ``'x'``) the alphabet in which
              ``self`` is to be expanded

            OUTPUT:

            - an expansion of ``self`` into the ``n`` non-commuting
              variables specified by ``alphabet``

            EXAMPLES::

                sage: h = SymmetricFunctionsNonCommutingVariables(QQ).h()
                sage: h[[1,3],[2]].expand(3)
                2*x0^3 + x0^2*x1 + x0^2*x2 + 2*x0*x1*x0 + x0*x1^2 + x0*x1*x2 + 2*x0*x2*x0
                 + x0*x2*x1 + x0*x2^2 + x1*x0^2 + 2*x1*x0*x1 + x1*x0*x2 + x1^2*x0 + 2*x1^3
                 + x1^2*x2 + x1*x2*x0 + 2*x1*x2*x1 + x1*x2^2 + x2*x0^2 + x2*x0*x1 + 2*x2*x0*x2
                 + x2*x1*x0 + x2*x1^2 + 2*x2*x1*x2 + x2^2*x0 + x2^2*x1 + 2*x2^3
                sage: x = SymmetricFunctionsNonCommutingVariables(QQ).x()
                sage: x[[1,3],[2]].expand(3)
                -x0^2*x1 - x0^2*x2 - x0*x1^2 - x0*x1*x2 - x0*x2*x1 - x0*x2^2 - x1*x0^2
                 - x1*x0*x2 - x1^2*x0 - x1^2*x2 - x1*x2*x0 - x1*x2^2 - x2*x0^2 - x2*x0*x1
                 - x2*x1*x0 - x2*x1^2 - x2^2*x0 - x2^2*x1
            """
            m = self.parent().realization_of().monomial()
            return m(self).expand(n, alphabet)

        def to_symmetric_function(self):
            r"""
            Compute the projection of an element of symmetric function in
            non-commuting variables to the symmetric functions.

            The projection of a monomial symmetric function in non-commuting
            variables indexed by the set partition ``A`` is defined as

            .. MATH::

                \mathbf{m}_A \mapsto m_{\lambda(A)} \prod_i n_i(\lambda(A))!

            where `\lambda(A)` is the partition associated with `A` by
            taking the sizes of the parts and `n_i(\mu)` is the
            multiplicity of `i` in `\mu`.  For other bases this map is extended
            linearly.

            OUTPUT: an element of the symmetric functions in the monomial basis

            EXAMPLES::

                sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
                sage: e = NCSym.e()
                sage: h = NCSym.h()
                sage: p = NCSym.p()
                sage: cp = NCSym.cp()
                sage: x = NCSym.x()
                sage: cp[[1,3],[2]].to_symmetric_function()
                m[2, 1]
                sage: x[[1,3],[2]].to_symmetric_function()
                -6*m[1, 1, 1] - 2*m[2, 1]
                sage: e[[1,3],[2]].to_symmetric_function()
                2*e[2, 1]
                sage: h[[1,3],[2]].to_symmetric_function()
                2*h[2, 1]
                sage: p[[1,3],[2]].to_symmetric_function()
                p[2, 1]
            """
            m = self.parent().realization_of().monomial()
            return m(self).to_symmetric_function()

        def to_wqsym(self):
            r"""
            Return the image of ``self`` under the canonical
            inclusion map `NCSym \to WQSym`.

            The canonical inclusion map `NCSym \to WQSym` is
            an injective homomorphism of algebras. It sends a
            basis element `\mathbf{m}_A` of `NCSym` to the sum of
            basis elements `\mathbf{M}_P` of `WQSym`, where `P`
            ranges over all ordered set partitions that become
            `A` when the ordering is forgotten.
            This map is denoted by `\theta` in [BZ05]_ (17).

            .. SEEALSO::

                :class:`WordQuasiSymmetricFunctions` for a
                definition of `WQSym`.

            EXAMPLES::

                sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
                sage: e = NCSym.e()
                sage: h = NCSym.h()
                sage: p = NCSym.p()
                sage: cp = NCSym.cp()
                sage: x = NCSym.x()
                sage: m = NCSym.m()
                sage: m[[1,3],[2]].to_wqsym()
                M[{1, 3}, {2}] + M[{2}, {1, 3}]
                sage: x[[1,3],[2]].to_wqsym()
                -M[{1}, {2}, {3}] - M[{1}, {2, 3}] - M[{1}, {3}, {2}]
                 - M[{1, 2}, {3}] - M[{2}, {1}, {3}] - M[{2}, {3}, {1}]
                 - M[{2, 3}, {1}] - M[{3}, {1}, {2}] - M[{3}, {1, 2}]
                 - M[{3}, {2}, {1}]
                sage: (4*p[[1,3],[2]]-p[[1]]).to_wqsym()
                -M[{1}] + 4*M[{1, 2, 3}] + 4*M[{1, 3}, {2}] + 4*M[{2}, {1, 3}]
            """
            parent = self.parent()
            NCSym = parent.realization_of()
            R = parent.base_ring()
            m = NCSym.monomial()
            from sage.combinat.chas.wqsym import WordQuasiSymmetricFunctions
            M = WordQuasiSymmetricFunctions(R).M()
            from itertools import permutations
            OSP = M.basis().keys()

            def to_wqsym_on_m_basis(A):
                # Return the image of `\mathbf{m}_A` under the inclusion
                # map `NCSym \to WQSym`.
                l = len(A)
                return M.sum_of_terms(((OSP([A[ui] for ui in u]), 1)
                                       for u in permutations(range(l))),
                                      distinct=True)
            return M.linear_combination((to_wqsym_on_m_basis(A), coeff)
                                        for A, coeff in m(self))

        def internal_coproduct(self):
            r"""
            Return the internal coproduct of ``self``.

            The internal coproduct is defined on the power sum basis as

            .. MATH::

                \mathbf{p}_A \mapsto \mathbf{p}_A \otimes \mathbf{p}_A

            and the map is extended linearly.

            OUTPUT: an element of the tensor square of the basis of ``self``

            EXAMPLES::

                sage: x = SymmetricFunctionsNonCommutingVariables(QQ).x()
                sage: x[[1,3],[2]].internal_coproduct()
                x{{1}, {2}, {3}} # x{{1, 3}, {2}} + x{{1, 3}, {2}} # x{{1}, {2}, {3}}
                 + x{{1, 3}, {2}} # x{{1, 3}, {2}}
            """
            return self.parent().internal_coproduct(self)

        def omega(self):
            r"""
            Return the involution `\omega` applied to ``self``.

            The involution `\omega` is defined by

            .. MATH::

                \mathbf{e}_A \mapsto \mathbf{h}_A

            and the result is extended linearly.

            OUTPUT: an element in the same basis as ``self``

            EXAMPLES::

                sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
                sage: m = NCSym.m()
                sage: m[[1,3],[2]].omega()
                -2*m{{1, 2, 3}} - m{{1, 3}, {2}}
                sage: p = NCSym.p()
                sage: p[[1,3],[2]].omega()
                -p{{1, 3}, {2}}
                sage: cp = NCSym.cp()
                sage: cp[[1,3],[2]].omega()
                -2*cp{{1, 2, 3}} - cp{{1, 3}, {2}}
                sage: x = NCSym.x()
                sage: x[[1,3],[2]].omega()
                -2*x{{1}, {2}, {3}} - x{{1, 3}, {2}}
            """
            P = self.parent()
            e = P.realization_of().e()
            h = P.realization_of().h()
            return P(h.sum_of_terms(e(self)))


class MultiplicativeNCSymBases(Category_realization_of_parent):
    r"""
    Category of multiplicative bases of symmetric functions in non-commuting variables.

    A multiplicative basis is one for which `\mathbf{b}_A \mathbf{b}_B = \mathbf{b}_{A|B}`
    where `A|B` is the :meth:`~sage.combinat.set_partition.SetPartition.pipe` operation
    on set partitions.

    EXAMPLES::

        sage: from sage.combinat.ncsym.bases import MultiplicativeNCSymBases
        sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
        sage: MultiplicativeNCSymBases(NCSym)
        Category of multiplicative bases of symmetric functions in non-commuting variables over the Rational Field
    """

    def super_categories(self):
        r"""
        Return the super categories of bases of the Hopf dual of the
        symmetric functions in non-commuting variables.

        OUTPUT: list of categories

        TESTS::

            sage: from sage.combinat.ncsym.bases import MultiplicativeNCSymBases
            sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
            sage: MultiplicativeNCSymBases(NCSym).super_categories()
            [Category of bases of symmetric functions in non-commuting variables over the Rational Field]
        """
        return [NCSymBases(self.base())]

    def _repr_(self):
        r"""
        Return a string representation of ``self``.

        TESTS::

            sage: from sage.combinat.ncsym.bases import MultiplicativeNCSymBases
            sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
            sage: MultiplicativeNCSymBases(NCSym)
            Category of multiplicative bases of symmetric functions in non-commuting variables over the Rational Field
        """
        return "Category of multiplicative bases of symmetric functions in non-commuting"\
               " variables over the {}".format(self.base().base_ring())

    class ParentMethods:
        def product_on_basis(self, A, B):
            r"""
            The product on basis elements.

            The product on a multiplicative basis is given by
            `\mathbf{b}_A \cdot \mathbf{b}_B = \mathbf{b}_{A | B}`.

            The bases `\{ \mathbf{e}, \mathbf{h}, \mathbf{x}, \mathbf{cp}, \mathbf{p},
            \mathbf{chi}, \mathbf{rho} \}` are all multiplicative.

            INPUT:

            - ``A``, ``B`` -- set partitions

            OUTPUT: an element in the basis ``self``

            EXAMPLES::

                sage: e = SymmetricFunctionsNonCommutingVariables(QQ).e()
                sage: h = SymmetricFunctionsNonCommutingVariables(QQ).h()
                sage: x = SymmetricFunctionsNonCommutingVariables(QQ).x()
                sage: cp = SymmetricFunctionsNonCommutingVariables(QQ).cp()
                sage: p = SymmetricFunctionsNonCommutingVariables(QQ).p()
                sage: chi = SymmetricFunctionsNonCommutingVariables(QQ).chi()
                sage: rho = SymmetricFunctionsNonCommutingVariables(QQ).rho()
                sage: A = SetPartition([[1], [2, 3]])
                sage: B = SetPartition([[1], [3], [2,4]])
                sage: e.product_on_basis(A, B)
                e{{1}, {2, 3}, {4}, {5, 7}, {6}}
                sage: h.product_on_basis(A, B)
                h{{1}, {2, 3}, {4}, {5, 7}, {6}}
                sage: x.product_on_basis(A, B)
                x{{1}, {2, 3}, {4}, {5, 7}, {6}}
                sage: cp.product_on_basis(A, B)
                cp{{1}, {2, 3}, {4}, {5, 7}, {6}}
                sage: p.product_on_basis(A, B)
                p{{1}, {2, 3}, {4}, {5, 7}, {6}}
                sage: chi.product_on_basis(A, B)
                chi{{1}, {2, 3}, {4}, {5, 7}, {6}}
                sage: rho.product_on_basis(A, B)
                rho{{1}, {2, 3}, {4}, {5, 7}, {6}}
                sage: e.product_on_basis(A,B)==e(h(e(A))*h(e(B)))
                True
                sage: h.product_on_basis(A,B)==h(x(h(A))*x(h(B)))
                True
                sage: x.product_on_basis(A,B)==x(h(x(A))*h(x(B)))
                True
                sage: cp.product_on_basis(A,B)==cp(p(cp(A))*p(cp(B)))
                True
                sage: p.product_on_basis(A,B)==p(e(p(A))*e(p(B)))
                True
            """
            return self.monomial(A.pipe(B))

    class ElementMethods:
        pass


class NCSymDualBases(Category_realization_of_parent):
    r"""
    Category of bases of dual symmetric functions in non-commuting variables.

    EXAMPLES::

        sage: from sage.combinat.ncsym.bases import NCSymDualBases
        sage: DNCSym = SymmetricFunctionsNonCommutingVariables(QQ).dual()
        sage: NCSymDualBases(DNCSym)
        Category of bases of dual symmetric functions in non-commuting variables over the Rational Field
    """

    def super_categories(self):
        r"""
        Return the super categories of bases of the Hopf dual of the
        symmetric functions in non-commuting variables.

        OUTPUT: list of categories

        TESTS::

            sage: from sage.combinat.ncsym.bases import NCSymBases
            sage: NCSym = SymmetricFunctionsNonCommutingVariables(QQ)
            sage: NCSymBases(NCSym).super_categories()
            [Category of bases of NCSym or NCSym^* over the Rational Field]
        """
        return [NCSymOrNCSymDualBases(self.base())]

    def _repr_(self):
        r"""
        Return a string representation of ``self``.

        TESTS::

            sage: from sage.combinat.ncsym.bases import NCSymDualBases
            sage: DNCSym = SymmetricFunctionsNonCommutingVariables(QQ).dual()
            sage: NCSymDualBases(DNCSym)
            Category of bases of dual symmetric functions in non-commuting variables over the Rational Field
        """
        return "Category of bases of dual symmetric functions in non-commuting"\
               " variables over the {}".format(self.base().base_ring())



from sage.libs.gsl.fft cimport *

cdef class FastFourierTransform_base:
    pass

cdef class FastFourierTransform_complex(FastFourierTransform_base):
    cdef double * data
    cdef size_t n
    cdef size_t stride
    cdef gsl_fft_complex_wavetable * wt
    cdef gsl_fft_complex_workspace * mem
